# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import re
import sys
import struct
import subprocess

local_file_header = [
    ("signature", "uint32"),
    ("min_version", "uint16"),
    ("general_flag", "uint16"),
    ("compression", "uint16"),
    ("lastmod_time", "uint16"),
    ("lastmod_date", "uint16"),
    ("crc32", "uint32"),
    ("compressed_size", "uint32"),
    ("uncompressed_size", "uint32"),
    ("filename_size", "uint16"),
    ("extra_field_size", "uint16"),
    ("filename", "filename_size"),
    ("extra_field", "extra_field_size"),
    ("data", "compressed_size")
]

cdir_entry = [
    ("signature", "uint32"),
    ("creator_version", "uint16"),
    ("min_version", "uint16"),
    ("general_flag", "uint16"),
    ("compression", "uint16"),
    ("lastmod_time", "uint16"),
    ("lastmod_date", "uint16"),
    ("crc32", "uint32"),
    ("compressed_size", "uint32"),
    ("uncompressed_size", "uint32"),
    ("filename_size", "uint16"),
    ("extrafield_size", "uint16"),
    ("filecomment_size", "uint16"),
    ("disknum", "uint16"),
    ("internal_attr", "uint16"),
    ("external_attr", "uint32"),
    ("offset", "uint32"),
    ("filename", "filename_size"),
    ("extrafield", "extrafield_size"),
    ("filecomment", "filecomment_size"),
]

cdir_end = [
    ("signature", "uint32"),
    ("disk_num", "uint16"),
    ("cdir_disk", "uint16"),
    ("disk_entries", "uint16"),
    ("cdir_entries", "uint16"),
    ("cdir_size", "uint32"),
    ("cdir_offset", "uint32"),
    ("comment_size", "uint16"),
]

type_mapping = {"uint32": "I", "uint16": "H"}


def format_struct(format):
    string_fields = {}
    fmt = "<"
    for (name, value) in iter(format):
        try:
            fmt += type_mapping[value][0]
        except KeyError:
            string_fields[name] = value
    return (fmt, string_fields)


def size_of(format):
    return struct.calcsize(format_struct(format)[0])


class MyStruct:
    def __init__(self, format, string_fields):
        self.__dict__["struct_members"] = {}
        self.__dict__["format"] = format
        self.__dict__["string_fields"] = string_fields

    def addMember(self, name, value):
        self.__dict__["struct_members"][name] = value

    def __getattr__(self, item):
        try:
            return self.__dict__["struct_members"][item]
        except:
            pass
        print("no %s" % item)
        print(self.__dict__["struct_members"])
        raise AttributeError

    def __setattr__(self, item, value):
        if item in self.__dict__["struct_members"]:
            self.__dict__["struct_members"][item] = value
        else:
            raise AttributeError

    def pack(self):
        extra_data = ""
        values = []
        string_fields = self.__dict__["string_fields"]
        struct_members = self.__dict__["struct_members"]
        format = self.__dict__["format"]
        for (name, _) in format:
            if name in string_fields:
                extra_data = extra_data + struct_members[name]
            else:
                values.append(struct_members[name])
        return struct.pack(format_struct(format)[0], *values) + extra_data

ENDSIG = 0x06054b50


def assert_true(cond, msg):
    if not cond:
        raise Exception(msg)
        exit(1)


class BinaryBlob:
    def __init__(self, f):
        self.data = open(f, "rb").read()
        self.offset = 0
        self.length = len(self.data)

    def readAt(self, pos, length):
        self.offset = pos + length
        return self.data[pos:self.offset]

    def read_struct(self, format, offset=None):
        if offset == None:
            offset = self.offset
        (fstr, string_fields) = format_struct(format)
        size = struct.calcsize(fstr)
        data = self.readAt(offset, size)
        ret = struct.unpack(fstr, data)
        retstruct = MyStruct(format, string_fields)
        i = 0
        for (name, _) in iter(format):
            member_desc = None
            if not name in string_fields:
                member_data = ret[i]
                i = i + 1
            else:
                # zip has data fields which are described by other struct fields, this does
                # additional reads to fill em in
                member_desc = string_fields[name]
                member_data = self.readAt(self.offset, retstruct.__getattr__(member_desc))
            retstruct.addMember(name, member_data)
        # sanity check serialization code
        data = self.readAt(offset, self.offset - offset)
        out_data = retstruct.pack()
        assert_true(out_data == data, "Serialization fail %d !=%d" % (len(out_data), len(data)))
        return retstruct


def optimizejar(jar, outjar, inlog=None):
    if inlog is not None:
        inlog = open(inlog).read().rstrip()
        # in the case of an empty log still move the index forward
        if len(inlog) == 0:
            inlog = []
        else:
            inlog = inlog.split("\n")
    outlog = []
    jarblob = BinaryBlob(jar)
    dirend = jarblob.read_struct(cdir_end, jarblob.length - size_of(cdir_end))
    assert_true(dirend.signature == ENDSIG, "no signature in the end")
    cdir_offset = dirend.cdir_offset
    readahead = 0
    if inlog is None and cdir_offset == 4:
        readahead = struct.unpack("<I", jarblob.readAt(0, 4))[0]
        print("%s: startup data ends at byte %d" % (outjar, readahead))

    total_stripped = 0
    jarblob.offset = cdir_offset
    central_directory = []
    for i in range(0, dirend.cdir_entries):
        entry = jarblob.read_struct(cdir_entry)
        if entry.filename[-1:] == "/":
            total_stripped += len(entry.pack())
        else:
            total_stripped += entry.extrafield_size
        central_directory.append(entry)

    reordered_count = 0
    if inlog is not None:
        dup_guard = set()
        for ordered_name in inlog:
            if ordered_name in dup_guard:
                continue
            else:
                dup_guard.add(ordered_name)
            found = False
            for i in range(reordered_count, len(central_directory)):
                if central_directory[i].filename == ordered_name:
                    # swap the cdir entries
                    tmp = central_directory[i]
                    central_directory[i] = central_directory[reordered_count]
                    central_directory[reordered_count] = tmp
                    reordered_count = reordered_count + 1
                    found = True
                    break
            if not found:
                print("Can't find '%s' in %s" % (ordered_name, jar))

    outfd = open(outjar, "wb")
    out_offset = 0
    if inlog is not None:
        # have to put central directory at offset 4 cos 0 confuses some tools.
        # This also lets us specify how many entries should be preread
        dirend.cdir_offset = 4
        # make room for central dir + end of dir + 4 extra bytes at front
        out_offset = dirend.cdir_offset + dirend.cdir_size + size_of(cdir_end) - total_stripped
        outfd.seek(out_offset)

    cdir_data = ""
    written_count = 0
    crc_mapping = {}
    dups_found = 0
    dupe_bytes = 0
    # store number of bytes suggested for readahead
    for entry in central_directory:
        # read in the header twice..first for comparison, second time for convenience when writing out
        jarfile = jarblob.read_struct(local_file_header, entry.offset)
        assert_true(jarfile.filename == entry.filename, "Directory/Localheader mismatch")
        # drop directory entries
        if entry.filename[-1:] == "/":
            total_stripped += len(jarfile.pack())
            dirend.cdir_entries -= 1
            continue
        # drop extra field data
        else:
            total_stripped += jarfile.extra_field_size
        entry.extrafield = jarfile.extra_field = ""
        entry.extrafield_size = jarfile.extra_field_size = 0
        # January 1st, 2010
        entry.lastmod_date = jarfile.lastmod_date = ((2010 - 1980) << 9) | (1 << 5) | 1
        entry.lastmod_time = jarfile.lastmod_time = 0
        data = jarfile.pack()
        outfd.write(data)
        old_entry_offset = entry.offset
        entry.offset = out_offset
        out_offset = out_offset + len(data)
        entry_data = entry.pack()
        cdir_data += entry_data
        expected_len = entry.filename_size + entry.extrafield_size + entry.filecomment_size
        assert_true(len(entry_data) != expected_len,
                    "%s entry size - expected:%d got:%d" % (entry.filename, len(entry_data), expected_len))
        written_count += 1

        if entry.crc32 in crc_mapping:
            dups_found += 1
            dupe_bytes += entry.compressed_size + len(data) + len(entry_data)
            print("%s\n\tis a duplicate of\n%s\n---" % (entry.filename, crc_mapping[entry.crc32]))
        else:
            crc_mapping[entry.crc32] = entry.filename

        if inlog is not None:
            if written_count == reordered_count:
                readahead = out_offset
                print("%s: startup data ends at byte %d" % (outjar, readahead))
            elif written_count < reordered_count:
                pass
                #print("%s @ %d" % (entry.filename, out_offset))
        elif readahead >= old_entry_offset + len(data):
            outlog.append(entry.filename)
            reordered_count += 1

    if inlog is None:
        dirend.cdir_offset = out_offset

    if dups_found > 0:
        print("WARNING: Found %d duplicate files taking %d bytes" % (dups_found, dupe_bytes))

    dirend.cdir_size = len(cdir_data)
    dirend.disk_entries = dirend.cdir_entries
    dirend_data = dirend.pack()
    assert_true(size_of(cdir_end) == len(dirend_data), "Failed to serialize directory end correctly. Serialized size;%d, expected:%d" % (len(dirend_data), size_of(cdir_end)))

    outfd.seek(dirend.cdir_offset)
    outfd.write(cdir_data)
    outfd.write(dirend_data)

    # for ordered jars the central directory is written in the begining of the file, so a second central-directory
    # entry has to be written in the end of the file
    if inlog is not None:
        outfd.seek(0)
        outfd.write(struct.pack("<I", readahead))
        outfd.seek(out_offset)
        outfd.write(dirend_data)

    print "Stripped %d bytes" % total_stripped
    print "%s %d/%d in %s" % (("Ordered" if inlog is not None else "Deoptimized"),
                              reordered_count, len(central_directory), outjar)
    outfd.close()
    return outlog


if len(sys.argv) != 5:
    print "Usage: --optimize|--deoptimize %s JAR_LOG_DIR IN_JAR_DIR OUT_JAR_DIR" % sys.argv[0]
    exit(1)

jar_regex = re.compile("\\.jar?$")


def optimize(JAR_LOG_DIR, IN_JAR_DIR, OUT_JAR_DIR):
    ls = os.listdir(IN_JAR_DIR)
    for jarfile in ls:
        if not re.search(jar_regex, jarfile):
            continue
        injarfile = os.path.join(IN_JAR_DIR, jarfile)
        outjarfile = os.path.join(OUT_JAR_DIR, jarfile)
        logfile = os.path.join(JAR_LOG_DIR, jarfile + ".log")
        if not os.path.isfile(logfile):
            logfile = None
        optimizejar(injarfile, outjarfile, logfile)


def deoptimize(JAR_LOG_DIR, IN_JAR_DIR, OUT_JAR_DIR):
    if not os.path.exists(JAR_LOG_DIR):
        os.makedirs(JAR_LOG_DIR)

    ls = os.listdir(IN_JAR_DIR)
    for jarfile in ls:
        if not re.search(jar_regex, jarfile):
            continue
        injarfile = os.path.join(IN_JAR_DIR, jarfile)
        outjarfile = os.path.join(OUT_JAR_DIR, jarfile)
        logfile = os.path.join(JAR_LOG_DIR, jarfile + ".log")
        log = optimizejar(injarfile, outjarfile, None)
        open(logfile, "wb").write("\n".join(log))


def main():
    MODE = sys.argv[1]
    JAR_LOG_DIR = sys.argv[2]
    IN_JAR_DIR = sys.argv[3]
    OUT_JAR_DIR = sys.argv[4]
    if MODE == "--optimize":
        optimize(JAR_LOG_DIR, IN_JAR_DIR, OUT_JAR_DIR)
    elif MODE == "--deoptimize":
        deoptimize(JAR_LOG_DIR, IN_JAR_DIR, OUT_JAR_DIR)
    else:
        print("Unknown mode %s" % MODE)
        exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import re
import os
import shutil
import logging
import tempfile
import argparse
import ConfigParser
from argparse import ArgumentDefaultsHelpFormatter
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper
from util.b2g_helper import B2GHelper

logger = logging.getLogger(__name__)


class BackupRestoreHelper(object):
    """
    Workaround for backing up and restoring Firefox OS profiles. (BETA)
    """

    def __init__(self):
        self._FILE_PROFILE_INI = 'profiles.ini'
        self._FILE_COMPATIBILITY_INI = 'compatibility.ini'
        self._FILE_PERF_JS = 'prefs.js'
        self._LOCAL_DIR_SDCARD = 'sdcard'
        self._LOCAL_DIR_WIFI = 'wifi'
        self._LOCAL_FILE_WIFI = 'wifi/wpa_supplicant.conf'
        self._LOCAL_DIR_B2G = 'b2g-mozilla'
        self._LOCAL_DIR_DATA = 'data-local'
        self._LOCAL_DIR_DATA_APPS = 'webapps'
        self._REMOTE_DIR_SDCARD = '/sdcard/'
        self._REMOTE_FILE_WIFI = '/data/misc/wifi/wpa_supplicant.conf'
        self._REMOTE_FILE_WIFI_OWNER = 'system:wifi'
        self._REMOTE_DIR_B2G = '/data/b2g/mozilla'
        self._REMOTE_DIR_DATA = '/data/local'
        # default settings
        self.serial = None
        self.backup = False
        self.restore = False
        self.sdcard = False
        self.no_reboot = False
        self.profile_dir = 'mozilla-profile'
        self.skip_version_check = False

    def set_serial(self, serial):
        """
        Setup the serial number.
        @param serial: the given serial number.
        """
        self.serial = serial
        logger.debug('Set serial: {}'.format(self.serial))

    def set_backup(self, flag):
        """
        Setup the backup flag. If backup is enabled, then resotre will be disable.
        @param flag: True or Flase.
        """
        if flag:
            self.backup = True
            logger.debug('Set backup: {}'.format(self.backup))
            if self.restore:
                self.restore = False
                logger.debug('Set restore: {}, due to backup is enabled'.format(self.restore))
        else:
            self.backup = False
            logger.debug('Set backup: {}'.format(self.backup))

    def set_restore(self, flag):
        """
        Setup the restore flag. If restore is enabled, then backup will be disable.
        @param flag: True or Flase.
        """
        if flag:
            self.restore = True
            logger.debug('Set restore: {}'.format(self.restore))
            if self.backup:
                self.backup = False
                logger.debug('Set backup: {}, due to restore is enabled'.format(self.backup))
        else:
            self.restore = False
            logger.debug('Set restore: {}'.format(self.restore))

    def set_sdcard(self, flag):
        """
        Setup backup or resotre SDCard flag.
        @param flag: True or Flase.
        """
        self.sdcard = flag
        logger.debug('Set sdcard: {}'.format(self.sdcard))

    def set_no_reboot(self, flag):
        """
        Setup no_reboot flag.
        @param flag: True or Flase.
        """
        self.no_reboot = flag
        logger.debug('Set no_reboot: {}'.format(self.no_reboot))

    def set_profile_dir(self, profile_dir):
        """
        Setup profile_dir path.
        @param profile_dir: The path of profile folder.
        """
        self.profile_dir = profile_dir
        logger.debug('Set profile_dir: {}'.format(self.profile_dir))

    def set_skip_version_check(self, flag):
        """
        Setup skip_version_check flag.
        @param flag: True or Flase.
        """
        self.skip_version_check = flag
        logger.debug('Set skip_version_check: {}'.format(self.skip_version_check))

    def cli(self):
        """
        Handle the argument parse, and the return the instance itself.
        """
        # argument parser
        arg_parser = argparse.ArgumentParser(
            description='Workaround for backing up and restoring Firefox OS profiles. (BETA)',
            formatter_class=ArgumentDefaultsHelpFormatter)
        arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None,
                                help='Directs command to the device or emulator with the given serial number. '
                                     'Overrides ANDROID_SERIAL environment variable.')
        br_group = arg_parser.add_mutually_exclusive_group(required=True)
        br_group.add_argument('-b', '--backup', action='store_true', dest='backup', default=False,
                              help='Backup user profile.')
        br_group.add_argument('-r', '--restore', action='store_true', dest='restore', default=False,
                              help='Restore user profile.')
        arg_parser.add_argument('--sdcard', action='store_true', dest='sdcard', default=False,
                                help='Also backup/restore SD card.')
        arg_parser.add_argument('--no-reboot', action='store_true', dest='no_reboot', default=False,
                                help='Do not reboot B2G after backup/restore.')
        arg_parser.add_argument('-p', '--profile-dir', action='store', dest='profile_dir', default='mozilla-profile',
                                help='Specify the profile folder.')
        arg_parser.add_argument('--skip-version-check', action='store_true', dest='skip_version_check', default=False,
                                help='Turn off version check between backup profile and device.')
        arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                                help='Turn on verbose output, with all the debug logger.')

        # parse args and setup the logging
        args = arg_parser.parse_args()
        # setup the logging config
        if args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        # check ADB
        AdbWrapper.check_adb()
        # assign the variable
        self.set_serial(args.serial)
        if args.backup:
            self.set_backup(args.backup)
        elif args.restore:
            self.set_restore(args.restore)
        self.set_sdcard(args.sdcard)
        self.set_no_reboot(args.no_reboot)
        self.set_profile_dir(args.profile_dir)
        self.set_skip_version_check(args.skip_version_check)
        # return instance
        return self

    def backup_sdcard(self, local_dir, serial=None):
        """
        Backup data from device's SDCard to local folder.

        @param local_dir: the target local folder, will store data from device's SDCard to this folder.
        """
        logger.info('Backing up SD card...')
        # try to get the /sdcard folder on device
        output, retcode = AdbWrapper.adb_shell('ls -d {0}; echo $?'.format(self._REMOTE_DIR_SDCARD), serial=serial)
        output_list = [item for item in re.split(r'\n+', re.sub(r'\r+', '', output)) if item]
        ret_code = output_list[-1]
        output_list.remove(output_list[-1])
        ret_msg = '\n'.join(output_list)
        if ret_code == '0':
            target_dir = os.path.join(local_dir, self._LOCAL_DIR_SDCARD)
            os.makedirs(target_dir)
            logger.info('Backup: {0} to {1}'.format(self._REMOTE_DIR_SDCARD, target_dir))
            try:
                AdbWrapper.adb_pull(self._REMOTE_DIR_SDCARD, target_dir, serial=serial)
            except Exception as e:
                logger.debug(e)
                logger.error('Can not pull files from {0} to {1}.'.format(self._REMOTE_DIR_SDCARD, target_dir))
            logger.info('Backup SD card done.')
        else:
            logger.info(ret_msg)

    def restore_sdcard(self, local_dir, serial=None):
        """
        Restore data from local folder to device's SDCard.

        @param local_dir: the source local folder, will get data from this folder and than restore to device's SDCard.
        """
        logger.info('Restoring SD card...')
        target_dir = os.path.join(local_dir, self._LOCAL_DIR_SDCARD)
        if os.path.isdir(target_dir):
            logger.info('Restore: {0} to {1}'.format(target_dir, self._REMOTE_DIR_SDCARD))
            try:
                AdbWrapper.adb_push(target_dir, self._REMOTE_DIR_SDCARD, serial=serial)
            except Exception as e:
                logger.debug(e)
                logger.error('Can not push files from {0} to {1}.'.format(target_dir, self._REMOTE_DIR_SDCARD))
            logger.info('Restore SD card done.')
        else:
            logger.info('{0}: No such file or directory'.format(target_dir))

    def backup_profile(self, local_dir, serial=None):
        """
        Backup B2G user profile from device to local folder.

        @param local_dir: the target local folder, the backup data will store to this folder.
        @param serial: device serial number. (optional)
        """
        logger.info('Backing up profile...')
        # Backup Wifi
        wifi_dir = os.path.join(local_dir, self._LOCAL_DIR_WIFI)
        wifi_file = os.path.join(local_dir, self._LOCAL_FILE_WIFI)
        os.makedirs(wifi_dir)
        logger.info('Backing up Wifi information...')
        try:
            AdbWrapper.adb_pull(self._REMOTE_FILE_WIFI, wifi_file, serial=serial)
        except Exception as e:
            logger.debug(e)
            logger.error('If you don\'t have root permission, you cannot backup Wifi information.')
        # Backup profile
        b2g_mozilla_dir = os.path.join(local_dir, self._LOCAL_DIR_B2G)
        os.makedirs(b2g_mozilla_dir)
        logger.info('Backing up {0} to {1} ...'.format(self._REMOTE_DIR_B2G, b2g_mozilla_dir))
        try:
            AdbWrapper.adb_pull(self._REMOTE_DIR_B2G, b2g_mozilla_dir, serial=serial)
        except Exception as e:
            logger.debug(e)
            logger.error('Can not pull files from {0} to {1}'.format(self._REMOTE_DIR_B2G, b2g_mozilla_dir))
        # Backup data/local
        datalocal_dir = os.path.join(local_dir, self._LOCAL_DIR_DATA)
        os.makedirs(datalocal_dir)
        logger.info('Backing up {0} to {1} ...'.format(self._REMOTE_DIR_DATA, datalocal_dir))
        try:
            AdbWrapper.adb_pull(self._REMOTE_DIR_DATA, datalocal_dir, serial=serial)
        except Exception as e:
            logger.debug(e)
            logger.error('Can not pull files from {0} to {1}'.format(self._REMOTE_DIR_DATA, datalocal_dir))

        # remove gecko.mstone value, so that gecko can check the apps under /system/b2g/webapps again.
        # (Thanks SC Chien's help!)
        local_profile_path = self._get_profile_path(os.path.join(b2g_mozilla_dir, self._FILE_PROFILE_INI))
        local_perf = os.path.join(b2g_mozilla_dir, local_profile_path, self._FILE_PERF_JS)
        perf_contents = []
        with open(local_perf, 'r') as f:
            for line in f.readlines():
                if 'gecko.mstone' not in line:
                    perf_contents.append(line)
                else:
                    logger.debug('Remove from {}: {}'.format(local_perf, line))
        with open(local_perf, 'w') as f:
            for line in perf_contents:
                f.write(line)
        logger.info('Backup profile done.')

    @staticmethod
    def _compare_version(version_of_backup, version_of_device):
        """
        Compare two B2G versions.

        @param version_of_backup: the local backup profile's version string.
        @param version_of_device: the device profile's version string.

        @return: True if the backup profile's version less than or equal to device's version.

        @raise exception: if  the backup profile's version large than device's version.
        """
        version_of_backup_float = float(version_of_backup.split('.')[0])
        version_of_device_float = float(version_of_device.split('.')[0])
        if version_of_backup_float <= version_of_device_float:
            logger.info(
                'Backup Profile {} <= Device Profile {}'.format(version_of_backup_float, version_of_device_float))
            return True
        else:
            raise Exception(
                'Backup Profile {} > Device Profile {}'.format(version_of_backup_float, version_of_device_float))

    @staticmethod
    def _get_profile_path(ini_file_path):
        """
        Get the B2G profile's folder name from profile.ini file.
        (e.g. /data/b2g/mozilla/profiles.ini on device.)

        @param ini_file_path: the profile.ini file.

        @return: the profile folder name.

        @raise exception: if cannot get the profile folder name.
        """
        ini_config = ConfigParser.ConfigParser()
        try:
            ini_config.read(ini_file_path)
            profile_path = ini_config.get('Profile0', 'Path')
            logger.debug('Load [{}]: {}'.format(ini_file_path, ini_config._sections))
            return profile_path
        except:
            raise Exception(
                'Can not get profile path from [{}], content: {}'.format(ini_file_path, ini_config._sections))

    @staticmethod
    def _get_version_from_profile(ini_file_path):
        """
        Get the B2G last version from compatibility.ini file.
        (e.g. /data/b2g/mozilla/<PROFILE_FOLDER>/compatibility.ini on device.)

        @param ini_file_path: the compatibility.ini file.

        @return: the last version.

        @raise exception: if cannot get the version.
        """
        ini_config = ConfigParser.ConfigParser()
        try:
            ini_config.read(ini_file_path)
            last_version = ini_config.get('Compatibility', 'LastVersion')
            logger.debug('Load [{}]: {}'.format(ini_file_path, ini_config._sections))
            logger.info('The LastVersion of [{}] is [{}]'.format(ini_file_path, last_version))
            return last_version
        except:
            raise Exception(
                'Can not get last version from [{}], content: {}'.format(ini_file_path, ini_config._sections))

    def _check_profile_version(self, local_dir, serial=None):
        """
        Check the versions of backup and device.
        The lower backup can restore to device. However the higher backup cannot.

        @param local_dir: the local backup folder.
        @param serial: device serial number. (optional)

        @return: True if backup version is lower than device's.

        @raise exception: if cannot load profiles or versions.
        """
        if self.skip_version_check:
            logger.info('Skip version check.')
            return True
        logger.info('Checking profile...')
        # get local version
        if os.path.isdir(local_dir):
            local_profile_path = self._get_profile_path(
                os.path.join(local_dir, self._LOCAL_DIR_B2G, self._FILE_PROFILE_INI))
            version_of_backup = self._get_version_from_profile(
                os.path.join(local_dir, self._LOCAL_DIR_B2G, local_profile_path, self._FILE_COMPATIBILITY_INI))
        else:
            raise Exception('Can not load profile from [{}]'.format(os.path.abspath(local_dir)))
        tmp_dir = None
        try:
            # get remote version
            tmp_dir = tempfile.mkdtemp(prefix='backup_restore_')
            logger.debug('TEMP Folder for check profile: {}'.format(tmp_dir))
            try:
                AdbWrapper.adb_pull(os.path.join(self._REMOTE_DIR_B2G, self._FILE_PROFILE_INI), tmp_dir, serial=serial)
            except:
                raise Exception(
                    'Can not pull {2} from {0} to {1}. '
                    'Please run with --skip-version-check if you want to restore.'.format(
                        self._REMOTE_DIR_B2G, tmp_dir, self._FILE_PROFILE_INI))
            remote_profile_path = self._get_profile_path(os.path.join(tmp_dir, self._FILE_PROFILE_INI))
            try:
                AdbWrapper.adb_pull(
                    os.path.join(self._REMOTE_DIR_B2G, remote_profile_path, self._FILE_COMPATIBILITY_INI), tmp_dir,
                    serial=serial)
            except:
                raise Exception(
                    'Can not pull {2} from {0} to {1}. '
                    'Please run with --skip-version-check if you want to restore.'.format(
                        self._REMOTE_DIR_B2G, tmp_dir, self._FILE_COMPATIBILITY_INI))
            version_of_device = self._get_version_from_profile(
                os.path.join(os.path.join(tmp_dir, self._FILE_COMPATIBILITY_INI)))
            # compare
            return self._compare_version(version_of_backup, version_of_device)
        finally:
            if tmp_dir:
                logger.debug('Removing [{0}] folder...'.format(tmp_dir))
                shutil.rmtree(tmp_dir)
                logger.debug('TEMP Folder for check profile removed: {}'.format(tmp_dir))

    def restore_profile(self, local_dir, serial=None):
        """
        Restore B2G user profile from local folder to device.

        @param local_dir: the source local folder, the backup data will restore from this folder.
        @param serial: device serial number. (optional)
        """
        logger.info('Restoring profile...')
        if os.path.isdir(local_dir):
            # Restore Wifi
            wifi_file = os.path.join(local_dir, self._LOCAL_FILE_WIFI)
            if os.path.isfile(wifi_file):
                logger.info('Restoring Wifi information...')
                try:
                    AdbWrapper.adb_push(wifi_file, self._REMOTE_FILE_WIFI, serial=serial)
                except Exception as e:
                    logger.debug(e)
                    logger.error('If you don\'t have root permission, you cannot restore Wifi information.')
                AdbWrapper.adb_shell('chown {0} {1}'.format(self._REMOTE_FILE_WIFI_OWNER, self._REMOTE_FILE_WIFI))
            # Restore profile
            b2g_mozilla_dir = os.path.join(local_dir, self._LOCAL_DIR_B2G)
            if os.path.isdir(b2g_mozilla_dir):
                logger.info('Restore from {0} to {1} ...'.format(b2g_mozilla_dir, self._REMOTE_DIR_B2G))
                AdbWrapper.adb_shell('rm -r {0}'.format(self._REMOTE_DIR_B2G))
                try:
                    AdbWrapper.adb_push(b2g_mozilla_dir, self._REMOTE_DIR_B2G, serial=serial)
                except Exception as e:
                    logger.debug(e)
                    logger.error('Can not push files from {0} to {1}'.format(b2g_mozilla_dir, self._REMOTE_DIR_B2G))
            # Restore data/local
            datalocal_dir = os.path.join(local_dir, self._LOCAL_DIR_DATA)
            if os.path.isdir(datalocal_dir):
                logger.info('Restore from {0} to {1} ...'.format(datalocal_dir, self._REMOTE_DIR_DATA))
                AdbWrapper.adb_shell('rm -r {0}'.format(self._REMOTE_DIR_DATA))
                try:
                    AdbWrapper.adb_push(datalocal_dir, self._REMOTE_DIR_DATA, serial=serial)
                except Exception as e:
                    logger.debug(e)
                    logger.error('Can not push files from {0} to {1}'.format(datalocal_dir, self._REMOTE_DIR_DATA))
            logger.info('Restore profile done.')
        else:
            logger.info('{0}: No such file or directory'.format(local_dir))

    def run(self):
        """
        Entry point.
        """
        # get the device's serial number
        devices = AdbWrapper.adb_devices()
        if len(devices) == 0:
            raise Exception('No device.')
        else:
            self.serial = AdbHelper.get_serial(self.serial)
            if self.serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                logger.debug('Setup serial to [{0}]'.format(self.serial))

        # checking the adb root for backup/restore
        if not AdbWrapper.adb_root(serial=self.serial):
            raise Exception('No root permission for backup and resotre.')

        if self.serial:
            logger.info('Target device [{0}]'.format(self.serial))
        # Backup
        if self.backup:
            tmp_dir = None
            try:
                # Create temp folder
                tmp_dir = tempfile.mkdtemp(prefix='backup_restore_')
                logger.debug('TEMP Foler: {}'.format(tmp_dir))
                # check the local profile folder
                if os.path.isdir(self.profile_dir):
                    if len(os.listdir(self.profile_dir)) == 0:
                        logger.debug('{} is empty. Clean it.')
                        shutil.rmtree(self.profile_dir)
                    else:
                        raise Exception(
                            '[{0}] folder already exists. Please check again.'.format(os.path.abspath(self.profile_dir)))
                # Stop B2G
                B2GHelper.stop_b2g(serial=self.serial)
                # Backup User Profile
                self.backup_profile(local_dir=tmp_dir, serial=self.serial)
                # Backup SDCard
                if self.sdcard:
                    self.backup_sdcard(local_dir=tmp_dir, serial=self.serial)
                # Copy backup files from temp folder to target folder
                logger.info('Copy profile from [{0}] to [{1}].'.format(tmp_dir, self.profile_dir))
                shutil.copytree(tmp_dir, self.profile_dir)
                # Start B2G
                if not self.no_reboot:
                    B2GHelper.start_b2g(serial=self.serial)
            finally:
                if tmp_dir:
                    logger.debug('Removing [{0}] folder...'.format(tmp_dir))
                    shutil.rmtree(tmp_dir)
                    logger.debug('TEMP Folder for backup removed: {}'.format(tmp_dir))
        # Restore
        elif self.restore:
            # Checking the Version of Profile
            if self._check_profile_version(local_dir=self.profile_dir, serial=self.serial):
                # Stop B2G
                B2GHelper.stop_b2g(serial=self.serial)
                # Restore User Profile
                self.restore_profile(local_dir=self.profile_dir, serial=self.serial)
                # Restore SDCard
                if self.sdcard:
                    self.restore_sdcard(local_dir=self.profile_dir, serial=self.serial)
                # Start B2G
                if not self.no_reboot:
                    B2GHelper.start_b2g(serial=self.serial)
            else:
                logger.error('The version on device is smaller than backup\'s version.')


def main():
    try:
        BackupRestoreHelper().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import logging
from adb_helper import AdbWrapper


logger = logging.getLogger(__name__)


class B2GHelper(object):

    @classmethod
    def stop_b2g(cls, serial=None):
        """
        Stop B2G process.
        @return: The stdout and return code. e.g. (stdout, retcode)
        """
        logger.info('Stop B2G.')
        output, retcode = AdbWrapper.adb_shell('stop b2g', serial=serial)
        logger.debug('RetCode: {}, Stdout: {}'.format(retcode, output))

    @classmethod
    def start_b2g(cls, serial=None):
        """
        Start B2G process.
        @return: The stdout and return code. e.g. (stdout, retcode)
        """
        logger.info('Start B2G.')
        output, retcode = AdbWrapper.adb_shell('start b2g', serial=serial)
        logger.debug('RetCode: {}, Stdout: {}'.format(retcode, output))



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import re
import os
import json
import logging
import argparse
from argparse import ArgumentDefaultsHelpFormatter
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper

logger = logging.getLogger(__name__)


class CrashReporter(object):
    """
    Get the Crash Reports from Firefox OS Phone.
    """

    def __init__(self):
        self.pending_path = '/data/b2g/mozilla/Crash Reports/pending'
        self.submitted_path = '/data/b2g/mozilla/Crash Reports/submitted'
        self.pending_stdout = None
        self.submitted_stdout = None
        self.pending_files = []
        self.submitted_files = []
        self.submitted_url_list = []
        self.serial = None
        self.log_json = None

    def set_serial(self, serial):
        """
        Setup the serial number.
        @param serial: the given serial number.
        """
        self.serial = serial
        logger.debug('Set serial: {}'.format(self.serial))

    def set_log_json(self, log_json):
        """
        Setup the log_json file path.
        @param log_json: the outpupt json file path.
        """
        self.log_json = log_json
        logger.debug('Set log_json: {}'.format(self.log_json))

    def cli(self):
        """
        Handle the argument parse, and the return the instance itself.
        """
        # argument parser
        arg_parser = argparse.ArgumentParser(description='Get the Crash Reports from Firefox OS Phone.',
                                             formatter_class=ArgumentDefaultsHelpFormatter)
        arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None,
                                help='Directs command to the device or emulator with the given serial number. '
                                     'Overrides ANDROID_SERIAL environment variable.')
        arg_parser.add_argument('--log-json', action='store', dest='log_json', default=None, help='JSON ouptut.')
        arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                                help='Turn on verbose output, with all the debug logger.')
        # parse args and setup the logging
        args = arg_parser.parse_args()
        # setup the logging config
        if args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        # check ADB
        AdbWrapper.check_adb()
        # assign variable
        self.set_serial(args.serial)
        self.set_log_json(args.log_json)
        # return instance
        return self

    def get_crashreports(self, serial=None):
        """
        Print the pending and submitted crash reports on device.

        The submitted crashs report will be displayed with URL link.

        @param serial: device serial number. (optional)
        """
        AdbWrapper.adb_root(serial=serial)
        logger.info('Getting Crash Reports...')

        self.pending_stdout, retcode_pending = AdbWrapper.adb_shell('ls -al "{}"'.format(self.pending_path),
                                                             serial=serial)
        print('Pending Crash Reports:\n{}\n'.format(self.pending_stdout))

        self.submitted_stdout, retcode_submitted = AdbWrapper.adb_shell('ls -al "{}"'.format(self.submitted_path),
                                                                 serial=serial)
        print('Submitted Crash Reports:\n{}\n'.format(self.submitted_stdout))
        # parse stdout for getting filepath
        self.pending_files = self._parse_stdout(self.pending_path, self.pending_stdout)
        self.submitted_files = self._parse_stdout(self.submitted_path, self.submitted_stdout)

        self.submitted_url_list = []
        if retcode_submitted == 0:
            print('The links of Submitted Crash Reports:')
            for line in self.submitted_stdout.split('\n'):
                submmited_id = re.sub(r'\.txt\s*$', '', re.sub(r'^.+bp-', '', line))
                submitted_url = 'https://crash-stats.mozilla.com/report/index/{}'.format(submmited_id)
                self.submitted_url_list.append(submitted_url)
                print(submitted_url)

    def _parse_stdout(self, base_path='', stdout=''):
        if not stdout or 'No such file or directory' in stdout:
            return []
        else:
            result = []
            lines = stdout.replace('\r', '').split('\n')
            for line in lines:
                path = os.path.join(base_path, line.rsplit(' ', 1)[1])
                result.append(path)
                logger.debug('parse: {} to {}'.format(line, path))
            return result

    def get_crashreports_info_dict(self):
        """
        Get the Crash Reports information dict.
        @return: the list of Submitted URL.
        """
        return {'PendingCrashReportsStdout': self.pending_stdout,
                'SubmittedCrashReportsStdout': self.submitted_stdout,
                'PendingCrashReports': self.pending_files,
                'SubmittedCrashReports': self.submitted_files,
                'SubmittedUrl': self.submitted_url_list}

    def output_log(self):
        if self.log_json:
            with open(self.log_json, 'w') as f:
                result = self.get_crashreports_info_dict()
                json.dump(result, f, indent=4)

    def run(self):
        """
        Entry point.
        """
        devices = AdbWrapper.adb_devices()

        if len(devices) == 0:
            raise Exception('No device.')
        elif len(devices) >= 1:
            final_serial = AdbHelper.get_serial(self.serial)
            if final_serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                    self.get_crashreports(serial=final_serial)
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                print('Serial: {0} (State: {1})'.format(final_serial, devices[final_serial]))
                self.get_crashreports(serial=final_serial)
            self.output_log()


def main():
    try:
        CrashReporter().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
Manual testing the color and cursor, please just run following command.

B{$ python <PATH_TO>/console_utilities.py}
"""


import os
import sys


COLOR_BLACK = 'black'
COLOR_BLUE = 'blue'
COLOR_GREEN = 'green'
COLOR_AQUA = 'aqua'
COLOR_RED = 'red'
COLOR_PURPLE = 'purple'
COLOR_YELLOW = 'yellow'
COLOR_WHITE = 'white'
COLOR_GRAY = 'gray'
COLOR_LIGHT_BLUE = 'lightblue'
COLOR_LIGHT_GREEN = 'lightgreen'
COLOR_LIGHT_AQUA = 'lightaqua'
COLOR_LIGHT_RED = 'lightred'
COLOR_LIGHT_PURPLE = 'lightpurple'
COLOR_LIGHT_YELLOW = 'lightyellow'
COLOR_BRIGHT_WHITE = 'brightwhite'


if os.name == 'posix':
    POSIX_COLOR_RESET = '\x1b[0m'
    POSIX_FG_COLOR_TABLE = {COLOR_BLACK: '\x1b[30m',
                            COLOR_BLUE: '\x1b[34m',
                            COLOR_GREEN: '\x1b[32m',
                            COLOR_AQUA: '\x1b[36m',
                            COLOR_RED: '\x1b[31m',
                            COLOR_PURPLE: '\x1b[35m',
                            COLOR_YELLOW: '\x1b[33m',
                            COLOR_WHITE: '\x1b[37m',
                            COLOR_GRAY: '\x1b[1;30m',
                            COLOR_LIGHT_BLUE: '\x1b[1;34m',
                            COLOR_LIGHT_GREEN: '\x1b[1;32m',
                            COLOR_LIGHT_AQUA: '\x1b[1;36m',
                            COLOR_LIGHT_RED: '\x1b[1;31m',
                            COLOR_LIGHT_PURPLE: '\x1b[1;35m',
                            COLOR_LIGHT_YELLOW: '\x1b[1;33m',
                            COLOR_BRIGHT_WHITE: '\x1b[1;37m'
                            }
    POSIX_BG_COLOR_TABLE = {COLOR_BLACK: '\x1b[40m',
                            COLOR_BLUE: '\x1b[44m',
                            COLOR_GREEN: '\x1b[42m',
                            COLOR_AQUA: '\x1b[46m',
                            COLOR_RED: '\x1b[41m',
                            COLOR_PURPLE: '\x1b[45m',
                            COLOR_YELLOW: '\x1b[43m',
                            COLOR_WHITE: '\x1b[47m',
                            COLOR_GRAY: '\x1b[1;40m',
                            COLOR_LIGHT_BLUE: '\x1b[44m',
                            COLOR_LIGHT_GREEN: '\x1b[42m',
                            COLOR_LIGHT_AQUA: '\x1b[46m',
                            COLOR_LIGHT_RED: '\x1b[41m',
                            COLOR_LIGHT_PURPLE: '\x1b[45m',
                            COLOR_LIGHT_YELLOW: '\x1b[43m',
                            COLOR_BRIGHT_WHITE: '\x1b[47m'
                            }
elif os.name == 'nt':
    import ctypes
    from ctypes import wintypes
    # Winbase.h
    WIN32_STD_OUTPUT_HANDLE = -11
    # Wincon.h and http://msdn.microsoft.com/en-us/library/windows/desktop/ms682088(v=vs.85).aspx#_win32_character_attributes
    WIN32_FOREGROUND_BLUE = 0x01          # Text color contains blue.
    WIN32_FOREGROUND_GREEN = 0x02         # Text color contains green.
    WIN32_FOREGROUND_RED = 0x04           # Text color contains red.
    WIN32_FOREGROUND_INTENSITY = 0x08     # Text color is intensified.
    WIN32_BACKGROUND_BLUE = 0x10          # Background color contains blue.
    WIN32_BACKGROUND_GREEN = 0x20         # Background color contains green.
    WIN32_BACKGROUND_RED = 0x40           # Background color contains red.
    WIN32_BACKGROUND_INTENSITY = 0x80     # Background color is intensified.

    WIN32_FG_COLOR_TABLE = {COLOR_BLACK: 0x00,
                            COLOR_BLUE: WIN32_FOREGROUND_BLUE,
                            COLOR_GREEN: WIN32_FOREGROUND_GREEN,
                            COLOR_AQUA: WIN32_FOREGROUND_BLUE | WIN32_FOREGROUND_GREEN,
                            COLOR_RED: WIN32_FOREGROUND_RED,
                            COLOR_PURPLE: WIN32_FOREGROUND_BLUE | WIN32_FOREGROUND_RED,
                            COLOR_YELLOW: WIN32_FOREGROUND_GREEN | WIN32_FOREGROUND_RED,
                            COLOR_WHITE: WIN32_FOREGROUND_BLUE | WIN32_FOREGROUND_GREEN | WIN32_FOREGROUND_RED,
                            COLOR_GRAY: WIN32_FOREGROUND_INTENSITY,
                            COLOR_LIGHT_BLUE: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_BLUE,
                            COLOR_LIGHT_GREEN: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_GREEN,
                            COLOR_LIGHT_AQUA: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_BLUE | WIN32_FOREGROUND_GREEN,
                            COLOR_LIGHT_RED: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_RED,
                            COLOR_LIGHT_PURPLE: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_BLUE | WIN32_FOREGROUND_RED,
                            COLOR_LIGHT_YELLOW: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_GREEN | WIN32_FOREGROUND_RED,
                            COLOR_BRIGHT_WHITE: WIN32_FOREGROUND_INTENSITY | WIN32_FOREGROUND_BLUE | WIN32_FOREGROUND_GREEN | WIN32_FOREGROUND_RED
                         }
    WIN32_BG_COLOR_TABLE = {COLOR_BLACK: 0x00,
                            COLOR_BLUE: WIN32_BACKGROUND_BLUE,
                            COLOR_GREEN: WIN32_BACKGROUND_GREEN,
                            COLOR_AQUA: WIN32_BACKGROUND_BLUE | WIN32_BACKGROUND_GREEN,
                            COLOR_RED: WIN32_BACKGROUND_RED,
                            COLOR_PURPLE: WIN32_BACKGROUND_BLUE | WIN32_BACKGROUND_RED,
                            COLOR_YELLOW: WIN32_BACKGROUND_GREEN | WIN32_BACKGROUND_RED,
                            COLOR_WHITE: WIN32_BACKGROUND_BLUE | WIN32_BACKGROUND_GREEN | WIN32_BACKGROUND_RED,
                            COLOR_GRAY: WIN32_BACKGROUND_INTENSITY,
                            COLOR_LIGHT_BLUE: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_BLUE,
                            COLOR_LIGHT_GREEN: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_GREEN,
                            COLOR_LIGHT_AQUA: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_BLUE | WIN32_BACKGROUND_GREEN,
                            COLOR_LIGHT_RED: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_RED,
                            COLOR_LIGHT_PURPLE: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_BLUE | WIN32_BACKGROUND_RED,
                            COLOR_LIGHT_YELLOW: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_GREEN | WIN32_BACKGROUND_RED,
                            COLOR_BRIGHT_WHITE: WIN32_BACKGROUND_INTENSITY | WIN32_BACKGROUND_BLUE | WIN32_BACKGROUND_GREEN | WIN32_BACKGROUND_RED
                         }

    class _CONSOLE_CURSOR_INFO(ctypes.Structure):
        # Wincon.h
        _fields_ = [('size', ctypes.wintypes.DWORD),
                    ('visible', ctypes.wintypes.BOOL)]

    class _CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
        # Wincon.h
        _fields_ = [('size', ctypes.wintypes._COORD),
                    ('cursorPosition', ctypes.wintypes._COORD),
                    ('attributes', ctypes.wintypes.WORD),
                    ('window', ctypes.wintypes.SMALL_RECT),
                    ('maximumWindowSize', ctypes.wintypes._COORD)]

        def __str__(self):
            return '%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d' % (self.size.Y, self.size.X,
                                               self.cursorPosition.Y, self.cursorPosition.X,
                                               self.attributes,
                                               self.window.Top, self.window.Left, self.window.Bottom, self.window.Right,
                                               self.maximumWindowSize.Y, self.maximumWindowSize.X)

    def GetConsoleScreenBufferInfo():
        # Wincon.h
        handle = get_win32_std_handle()
        csbi = _CONSOLE_SCREEN_BUFFER_INFO()
        ctypes.windll.kernel32.GetConsoleScreenBufferInfo(handle, ctypes.byref(csbi))
        return csbi

    def SetConsoleTextAttribute(attributes):
        # Wincon.h
        handle = get_win32_std_handle()
        ctypes.windll.kernel32.SetConsoleTextAttribute(handle, attributes)

    def get_win32_std_handle():
        # Winbase.h
        return ctypes.windll.kernel32.GetStdHandle(WIN32_STD_OUTPUT_HANDLE)


def hide_cursor():
    """
    Hide the cursor.
    """
    if os.name == 'posix':
        sys.stdout.write("\033[?25l")
        sys.stdout.flush()
    elif os.name == 'nt':
        ci = _CONSOLE_CURSOR_INFO()
        handle = get_win32_std_handle()
        # Wincon.h
        ctypes.windll.kernel32.GetConsoleCursorInfo(handle, ctypes.byref(ci))
        ci.visible = False
        ctypes.windll.kernel32.SetConsoleCursorInfo(handle, ctypes.byref(ci))


def show_cursor():
    """
    Show the cursor.
    """
    if os.name == 'posix':
        sys.stdout.write("\033[?25h")
        sys.stdout.flush()
    elif os.name == 'nt':
        ci = _CONSOLE_CURSOR_INFO()
        handle = get_win32_std_handle()
        # Wincon.h
        ctypes.windll.kernel32.GetConsoleCursorInfo(handle, ctypes.byref(ci))
        ci.visible = True
        ctypes.windll.kernel32.SetConsoleCursorInfo(handle, ctypes.byref(ci))


def print_color(message, fg_color=None, bg_color=None, newline=True):
    """
    Print message with color.
    @param message: The message.
    @param fg_color: The foreground color. Default is None.
    @param bg_color: The background color. Default is None.
    @param newline: Print message with newline. Defa
    """
    if os.name == 'posix':
        result = ''
        if fg_color is not None and fg_color in POSIX_FG_COLOR_TABLE:
            result = result + format(POSIX_FG_COLOR_TABLE[fg_color])
        if bg_color is not None and bg_color in POSIX_BG_COLOR_TABLE:
            result = result + format(POSIX_BG_COLOR_TABLE[bg_color])
        result = result + message + POSIX_COLOR_RESET
        if newline:
            print result
        else:
            print result,
    elif os.name == 'nt':
        # get default csbi
        default = GetConsoleScreenBufferInfo().attributes
        default_fg = default & 0x0f
        default_bg = default & 0xf0
        fg = default_fg
        bg = default_bg
        if fg_color is not None and fg_color in WIN32_FG_COLOR_TABLE:
            fg = WIN32_FG_COLOR_TABLE[fg_color]
        if bg_color is not None and bg_color in WIN32_BG_COLOR_TABLE:
            bg = WIN32_BG_COLOR_TABLE[bg_color]
        SetConsoleTextAttribute(fg | bg)
        if newline:
            print message
        else:
            print message,
        SetConsoleTextAttribute(default)
    else:
        if newline:
            print message
        else:
            print message,


if __name__ == '__main__':
    # test cursor
    print '### Cursor Test ###'
    import time
    hide_cursor()
    print 'Test hide_cursor() for 3 sec...'
    time.sleep(3)
    show_cursor()
    print 'Test show_cursor() for 3 sec...'
    time.sleep(3)
    # test color
    print '\n### Color Test ###'
    print_color('FG ' + COLOR_BLUE, fg_color=COLOR_BLUE)
    print_color('FG ' + COLOR_GREEN, fg_color=COLOR_GREEN)
    print_color('FG ' + COLOR_RED, fg_color=COLOR_RED)
    print_color('BG ' + COLOR_BLUE, bg_color=COLOR_BLUE)
    print_color('BG ' + COLOR_GREEN, bg_color=COLOR_GREEN)
    print_color('BG ' + COLOR_RED, bg_color=COLOR_RED)
    print_color('FG ' + COLOR_LIGHT_AQUA + ', BG ' + COLOR_PURPLE, fg_color=COLOR_LIGHT_AQUA, bg_color=COLOR_PURPLE)
    print_color('FG ' + COLOR_PURPLE + ', BG ' + COLOR_BRIGHT_WHITE, fg_color=COLOR_PURPLE, bg_color=COLOR_BRIGHT_WHITE)
    print_color('FG ' + COLOR_LIGHT_YELLOW + ', BG ' + COLOR_BLUE, fg_color=COLOR_LIGHT_YELLOW, bg_color=COLOR_BLUE)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import logging
import urllib2
import console_utilities


logger = logging.getLogger(__name__)


class Downloader(object):

    def download(self, source_url, dest_folder, status_callback=None, progress_callback=None):
        try:
            console_utilities.hide_cursor()
            f = urllib2.urlopen(source_url)
            logger.info('Downloading {} ...'.format(os.path.basename(source_url)))
            self.ensure_folder(dest_folder)
            filename_with_path = os.path.join(dest_folder, os.path.basename(source_url))
            with open(filename_with_path, "wb") as local_file:
                total_size = int(f.info().getheader('Content-Length').strip())
                pc = 0
                chunk_size = 8192
                while 1:
                    chunk = f.read(chunk_size)
                    pc += len(chunk)
                    if not chunk:
                        break
                    if progress_callback:
                        progress_callback(current_byte=pc, total_size=total_size)
                    local_file.write(chunk)
            logger.info('Download to {}'.format(filename_with_path))
            console_utilities.show_cursor()
            return filename_with_path
        except urllib2.HTTPError as e:
            logger.error(e)
        except urllib2.URLError as e:
            logger.error(e)

    def ensure_folder(self, folder):
        if not os.path.isdir(folder):
            os.makedirs(folder)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import logging
import argparse
from argparse import ArgumentDefaultsHelpFormatter
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper

logger = logging.getLogger(__name__)


class PhoneReseter(object):
    """
    Reset Firefox OS Phone.
    """

    def __init__(self):
        self.serial = None

    def set_serial(self, serial):
        """
        Setup the serial number.
        @param serial: the given serial number.
        """
        self.serial = serial
        logger.debug('Set serial: {}'.format(self.serial))

    def cli(self):
        """
        Handle the argument parse, and the return the instance itself.
        """
        # argument parser
        arg_parser = argparse.ArgumentParser(description='Reset Firefox OS Phone.',
                                             formatter_class=ArgumentDefaultsHelpFormatter)
        arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None,
                                help='Directs command to the device or emulator with the given serial number.'
                                     'Overrides ANDROID_SERIAL environment variable.')
        arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                                help='Turn on verbose output, with all the debug logger.')

        # parse args and setup the logging
        args = arg_parser.parse_args()
        # setup the logging config
        if args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        # check ADB
        AdbWrapper.check_adb()
        # assign the variable
        self.set_serial(args.serial)
        # return instance
        return self

    @staticmethod
    def reset_phone(serial=None):
        """
        Reset the B2G device.
        @param serial: device serial number. (optional)
        @raise exception: When no root permission for reset device.
        """
        # checking the adb root for backup/restore
        if not AdbWrapper.adb_root(serial=serial):
            raise Exception('No root permission for reset device.')
        # starting to reset
        logger.info('Starting to Reset Firefox OS Phone...')
        AdbWrapper.adb_shell('rm -r /cache/*', serial=serial)
        AdbWrapper.adb_shell('mkdir /cache/recovery', serial=serial)
        AdbWrapper.adb_shell('echo "--wipe_data" > /cache/recovery/command', serial=serial)
        AdbWrapper.adb_shell('reboot recovery', serial=serial)
        logger.info('Reset Firefox OS Phone done.')

    def run(self):
        """
        Entry point.
        """
        devices = AdbWrapper.adb_devices()

        if len(devices) == 0:
            raise Exception('No device.')
        elif len(devices) >= 1:
            final_serial = AdbHelper.get_serial(self.serial)
            if final_serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                    self.reset_phone(serial=final_serial)
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                print('Serial: {0} (State: {1})'.format(final_serial, devices[final_serial]))
                self.reset_phone(serial=final_serial)


def main():
    try:
        PhoneReseter().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import shutil
import logging
import argparse
import tempfile
import textwrap
from argparse import RawTextHelpFormatter
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper
from util.b2g_helper import B2GHelper

logger = logging.getLogger(__name__)


class FullPrivilegeResetter(object):
    """
    Enable/disable Certified Apps Debugging.
    """

    def __init__(self):
        self.serial = None
        self.disable = False

    def set_serial(self, serial):
        """
        Setup the serial number.
        @param serial: the given serial number.
        """
        self.serial = serial
        logger.debug('Set serial: {}'.format(self.serial))

    def set_disable(self, flag):
        """
        Setup the disable flag.
        @param flag: True or False.
        """
        self.disable = flag
        logger.debug('Set disable: {}'.format(self.disable))

    def cli(self):
        """
        Handle the argument parse, and the return the instance itself.
        """
        # argument parser
        arg_parser = argparse.ArgumentParser(description='Enable/disable Certified Apps Debugging.',
                                             formatter_class=RawTextHelpFormatter,
                                             epilog=textwrap.dedent("""\
                                                  Please enable "ADB and Devtools" of device.
                                                  Ref:
                                                  - https://developer.mozilla.org/en-US/docs/Tools/WebIDE
                                                  - https://developer.mozilla.org/en-US/docs/Tools/WebIDE/Running_and_debugging_apps#Debugging_apps
                                                  """))
        arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None,
                                help=textwrap.dedent("""\
                                     Directs command to the device or emulator with the
                                     given serial number. Overrides ANDROID_SERIAL
                                     environment variable. (default: %(default)s)
                                     """))
        arg_parser.add_argument('--disable', action='store_true', dest='disable', default=False,
                                help='Disable the privileges. (default: %(default)s)')
        arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                                help=textwrap.dedent("""\
                                     Turn on verbose output, with all the debug logger.
                                     (default: %(default)s)
                                     """))
        # parse args and setup the logging
        args = arg_parser.parse_args()
        # setup the logging config
        if args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        # check ADB
        AdbWrapper.check_adb()
        # assign variable
        self.set_serial(args.serial)
        self.set_disable(args.disable)
        # return instance
        return self

    @staticmethod
    def setup_certapps(enable=True, serial=None):
        """
        Set the devtools permission for certapps.
        @param enable: True will turn on the permission. False will turn off the permission.
        @param serial: device serial number. (optional)
        @raise exception: When it cannot pulling/pushing the pref.js file of device.
        """
        AdbWrapper.adb_root(serial=serial)
        logger.info('{} Full Privilege for WebIDE...'.format('Enabling' if enable else 'Disabling'))

        need_restart = True
        tmp_dir = None
        try:
            tmp_dir = tempfile.mkdtemp(prefix='enablecertapps_')
            # get profile folder name xxx.default under /data/b2g/mozilla/
            profile_dir_name, retcode = AdbWrapper.adb_shell('ls /data/b2g/mozilla/ | grep default', serial=serial)
            device_src_file = os.path.join('/data/b2g/mozilla/', profile_dir_name, 'prefs.js')
            dest_temp_file = os.path.join(tmp_dir, 'prefs.js.origin')
            try:
                logger.info('Pulling prefs.js file...')
                AdbWrapper.adb_pull(device_src_file, dest_temp_file, serial=serial)
            except:
                raise Exception('Error pulling prefs.js file.')

            dest_file = os.path.join(tmp_dir, 'prefs.js')
            with open(dest_temp_file, 'r') as fr:
                with open(dest_file, 'w') as fw:
                    match = False
                    is_forbid = 'false' if enable else 'true'
                    logger.debug('is_forbid: [{}]'.format(is_forbid))
                    for line in fr:
                        if 'devtools.debugger.forbid-certified-apps' in line:
                            logger.debug('line: [{}] to [{}]'.format(line, is_forbid))
                            if is_forbid in line:
                                # do not need to restart if the setting isn't changed
                                logger.info(
                                    'The full privilege is already {}.'.format('enabled' if enable else 'disabled'))
                                need_restart = False
                                break
                            else:
                                logger.info('Changing setting of pref.js file...')
                                fw.write(
                                    'user_pref("devtools.debugger.forbid-certified-apps", {});\n'.format(is_forbid))
                            match = True
                        else:
                            fw.write(line)
                    if not match:
                        if not enable:
                            # the forbid is true when there is no setting
                            logger.info('The full privilege is already disabled.')
                            need_restart = False
                        else:
                            if need_restart:
                                # adding setting when there is no setting and need to enable certapps
                                logger.info('Adding setting of pref.js file...')
                                fw.write(
                                    'user_pref("devtools.debugger.forbid-certified-apps", {});\n'.format(is_forbid))
            if need_restart:
                B2GHelper.stop_b2g(serial=serial)
                try:
                    logger.info('Pushing prefs.js file...')
                    AdbWrapper.adb_push(dest_file, device_src_file, serial=serial)
                except:
                    raise Exception('Error pushing prefs.js file.')
        finally:
            if need_restart:
                B2GHelper.start_b2g(serial=serial)
            if tmp_dir:
                shutil.rmtree(tmp_dir)
                logger.debug('Remove {}.'.format(tmp_dir))

    def run(self):
        """
        Entry point.
        """
        devices = AdbWrapper.adb_devices()

        is_enable = not self.disable
        if len(devices) == 0:
            raise Exception('No device.')
        elif len(devices) >= 1:
            final_serial = AdbHelper.get_serial(self.serial)
            if final_serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                    self.setup_certapps(enable=is_enable, serial=final_serial)
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                print('Serial: {0} (State: {1})'.format(final_serial, devices[final_serial]))
                self.setup_certapps(enable=is_enable, serial=final_serial)


def main():
    try:
        FullPrivilegeResetter().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import re
import os
import json
import shutil
import logging
import zipfile
import tempfile
import argparse
import subprocess
from distutils import util
from datetime import datetime
from argparse import ArgumentDefaultsHelpFormatter
from util import console_utilities
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper

logger = logging.getLogger(__name__)


class VersionChecker(object):
    def __init__(self):
        self.devices = None
        self.device_info_list = []
        self.no_color = False
        self.serial = None
        self.log_text = None
        self.log_json = None

    def set_serial(self, serial):
        """
        Setup the serial number.
        @param serial: the given serial number.
        """
        self.serial = serial
        logger.debug('Set serial: {}'.format(self.serial))

    def set_no_color(self, flag):
        """
        Setup the no_color flag.
        @param flag: True or Flas.
        """
        self.no_color = flag
        logger.debug('Set no_color: {}'.format(self.no_color))

    def set_log_text(self, log_text):
        """
        Setup the log_text file path.
        @param log_text: the output text file path.
        """
        self.log_text = log_text
        logger.debug('Set log_text: {}'.format(self.log_text))

    def set_log_json(self, log_json):
        """
        Setup the log_json file path.
        @param log_json: the outpupt json file path.
        """
        self.log_json = log_json
        logger.debug('Set log_json: {}'.format(self.log_json))

    def cli(self):
        """
        Handle the argument parse, and the return the instance itself.
        """
        # argument parser
        arg_parser = argparse.ArgumentParser(description='Check the version information of Firefox OS.',
                                             formatter_class=ArgumentDefaultsHelpFormatter)
        arg_parser.add_argument('--no-color', action='store_true', dest='no_color', default=False,
                                help='Do not print with color. NO_COLOR will overrides this option.')
        arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None,
                                help='Directs command to the device or emulator with the given serial number. '
                                     'Overrides ANDROID_SERIAL environment variable.')
        arg_parser.add_argument('--log-text', action='store', dest='log_text', default=None, help='Text ouput.')
        arg_parser.add_argument('--log-json', action='store', dest='log_json', default=None, help='JSON output.')
        arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                                help='Turn on verbose output, with all the debug logger.')

        # parse args and setup the logging
        args = arg_parser.parse_args()
        # setup the logging config
        if args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        # check ADB
        AdbWrapper.check_adb()
        # assign variable
        self.set_no_color(args.no_color)
        self.set_serial(args.serial)
        self.set_log_text(args.log_text)
        self.set_log_json(args.log_json)
        # return instance
        return self

    @staticmethod
    def get_device_info(serial=None):
        """
        Get the device information, include Gaia Version, Gecko Version, and so on.
        @param serial: device serial number. (optional)
        @return: the information dict object.
        """
        tmp_dir = None
        try:
            tmp_dir = tempfile.mkdtemp(prefix='checkversions_')
            # pull data from device
            try:
                AdbWrapper.adb_pull('/system/b2g/omni.ja', tmp_dir, serial=serial)
            except Exception as e:
                logger.debug(e)
                logger.error('Error pulling Gecko file.')
            try:
                AdbWrapper.adb_pull('/data/local/webapps/settings.gaiamobile.org/application.zip', tmp_dir,
                                    serial=serial)
            except Exception as e:
                logger.debug(e)
                try:
                    AdbWrapper.adb_pull('/system/b2g/webapps/settings.gaiamobile.org/application.zip', tmp_dir,
                                        serial=serial)
                except Exception as e:
                    logger.debug(e)
                    logger.error('Error pulling Gaia file.')
            try:
                AdbWrapper.adb_pull('/system/b2g/application.ini', tmp_dir, serial=serial)
            except Exception as e:
                logger.debug(e)
                logger.error('Error pulling application.ini file.')
            # get Gaia info
            gaia_rev = 'n/a'
            gaia_date = 'n/a'
            application_zip_file = os.path.join(tmp_dir, 'application.zip')
            if os.path.isfile(application_zip_file):
                with open(application_zip_file, 'rb') as f:
                    z = zipfile.ZipFile(f)
                    z.extract('resources/gaia_commit.txt', tmp_dir)
            else:
                logger.warning('Can not find application.zip file.')
            gaiacommit_file = os.path.join(tmp_dir, 'resources/gaia_commit.txt')
            if os.path.isfile(gaiacommit_file):
                with open(gaiacommit_file, "r") as f:
                    gaia_rev = re.sub(r'\n+', '', f.readline())
                    gaia_date_sec_from_epoch = re.sub(r'\n+', '', f.readline())
                gaia_date = datetime.utcfromtimestamp(int(gaia_date_sec_from_epoch)).strftime('%Y-%m-%d %H:%M:%S')
            else:
                logger.warning('Can not get gaia_commit.txt file from application.zip file.')
            # deoptimize omni.ja for Gecko info
            gecko_rev = 'n/a'
            if os.path.isfile(os.path.join(tmp_dir, 'omni.ja')):
                deopt_dir = os.path.join(tmp_dir, 'deopt')
                deopt_file = os.path.join(deopt_dir, 'omni.ja')
                deopt_exec = os.path.join(tmp_dir, 'optimizejars.py')
                os.makedirs(deopt_dir)
                # TODO rewrite optimizejars.py if possible
                current_dir = os.path.dirname(os.path.abspath(__file__))
                current_exec = os.path.join(current_dir, 'misc', 'optimizejars.py')
                shutil.copyfile(current_exec, deopt_exec)
                cmd = 'python %s --deoptimize %s %s %s' % (deopt_exec, tmp_dir, tmp_dir, deopt_dir)
                p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                output = p.communicate()[0]
                logger.debug('optimizejars.py stdout: {}'.format(output))
                # unzip omni.ja to get Gecko info
                if os.path.isfile(deopt_file):
                    with open(deopt_file, 'rb') as f:
                        z = zipfile.ZipFile(f)
                        z.extract('chrome/toolkit/content/global/buildconfig.html', tmp_dir)
                else:
                    logger.warning('Can not deoptimize omni.ja file.')
                    gecko_rev = 'n/a'
                # get Gecko info from buildconfig.html file
                buildconfig_file = os.path.join(tmp_dir, 'chrome/toolkit/content/global/buildconfig.html')
                if os.path.isfile(buildconfig_file):
                    for line in open(buildconfig_file, "r"):
                        if re.search(r'Built from', line):
                            ret = re.findall(r'>(.*?)<', line)
                            gecko_rev = ret[1]
                            break
                else:
                    logger.warning('Can not get buildconfig.html file from omni.ja file.')
            else:
                print 'Can not find omni.ja file.'
            # get Gecko version, and B2G BuildID from application.ini file
            build_id = 0
            version = 0
            if os.path.isfile(os.path.join(tmp_dir, 'application.ini')):
                for line in open(os.path.join(tmp_dir, 'application.ini'), "r"):
                    if re.search(r'^\s*BuildID', line):
                        ret = re.findall(r'.*?=(.*)', line)
                        build_id = ret[0]
                    if re.search(r'^\s*Version', line):
                        ret = re.findall(r'.*?=(.*)', line)
                        version = ret[0]
            else:
                build_id = 'n/a'
                version = 'n/a'
            # get device information by getprop command
            device_name = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.product.device', serial=serial)[0])
            firmware_release = re.sub(r'\r+|\n+', '',
                                      AdbWrapper.adb_shell('getprop ro.build.version.release', serial=serial)[0])
            firmware_incremental = re.sub(r'\r+|\n+', '',
                                          AdbWrapper.adb_shell('getprop ro.build.version.incremental', serial=serial)[
                                              0])
            firmware_date = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.build.date', serial=serial)[0])
            firmware_bootloader = re.sub(r'\r+|\n+', '',
                                         AdbWrapper.adb_shell('getprop ro.boot.bootloader', serial=serial)[0])
            # prepare the return information
            device_info = {'Serial': serial,
                           'Build ID': build_id,
                           'Gaia Revision': gaia_rev,
                           'Gaia Date': gaia_date,
                           'Gecko Revision': gecko_rev,
                           'Gecko Version': version,
                           'Device Name': device_name,
                           'Firmware(Release)': firmware_release,
                           'Firmware(Incremental)': firmware_incremental,
                           'Firmware Date': firmware_date,
                           'Bootloader': firmware_bootloader}
        finally:
            if tmp_dir:
                shutil.rmtree(tmp_dir)
                logger.debug('Remove {}.'.format(tmp_dir))
        return device_info

    @staticmethod
    def _print_device_info_item(title, value, title_color=None, value_color=None):
        console_utilities.print_color('{0:22s}'.format(title), fg_color=title_color, newline=False)
        console_utilities.print_color(value, fg_color=value_color)

    def print_device_info(self, device_info, no_color=False):
        """
        Print the device information.
        @param device_info: The information dict object.
        @param no_color: Print with color. Default is False.
        """
        # setup the format by platform
        if no_color:
            title_color = None
            sw_color = None
            hw_color = None
        else:
            title_color = console_utilities.COLOR_LIGHT_BLUE
            sw_color = console_utilities.COLOR_LIGHT_GREEN
            hw_color = console_utilities.COLOR_LIGHT_YELLOW
        # print the device information
        self._print_device_info_item('Build ID', device_info['Build ID'], title_color=title_color, value_color=sw_color)
        self._print_device_info_item('Gaia Revision', device_info['Gaia Revision'], title_color=title_color,
                                     value_color=sw_color)
        self._print_device_info_item('Gaia Date', device_info['Gaia Date'], title_color=title_color,
                                     value_color=sw_color)
        self._print_device_info_item('Gecko Revision', device_info['Gecko Revision'], title_color=title_color,
                                     value_color=sw_color)
        self._print_device_info_item('Gecko Version', device_info['Gecko Version'], title_color=title_color,
                                     value_color=sw_color)
        self._print_device_info_item('Device Name', device_info['Device Name'], title_color=title_color,
                                     value_color=hw_color)
        self._print_device_info_item('Firmware(Release)', device_info['Firmware(Release)'], title_color=title_color,
                                     value_color=hw_color)
        self._print_device_info_item('Firmware(Incremental)', device_info['Firmware(Incremental)'],
                                     title_color=title_color, value_color=hw_color)
        self._print_device_info_item('Firmware Date', device_info['Firmware Date'], title_color=title_color,
                                     value_color=hw_color)
        if device_info['Bootloader'] is not '':
            self._print_device_info_item('Bootloader', device_info['Bootloader'], title_color=title_color,
                                         value_color=hw_color)
        print ''

    def _output_log(self):
        """
        Write the information into file.
        Enable it by I{--log-text} and I{--log-json} arguments.
        """
        if self.log_json is None and self.log_text is None:
            return
        # prepare the result dict for parsing
        result = self.get_output_dict()
        # output
        if self.log_text is not None:
            with open(self.log_text, 'w') as outfile:
                for device_serial, device_info in result.items():
                    outfile.write('# %s\n' % device_serial)
                    if 'Skip' in device_info and device_info['Skip'] is True:
                        outfile.write('%s=%s\n' % ('Skip', device_info['Skip']))
                    else:
                        for key, value in device_info.items():
                            outfile.write('%s=%s\n' % (re.sub(r'\s+|\(|\)', '', key), re.sub(r'\s+', '_', value)))
                        outfile.write('\n')
        if self.log_json is not None:
            with open(self.log_json, 'w') as outfile:
                json.dump(result, outfile, indent=4)

    def get_output_dict(self):
        """
        Can get the devices' information dict object after run().
        """
        result = {}
        unknown_serial_index = 1
        for device_info in self.device_info_list:
            if device_info['Serial'] is None:
                device_serial = 'unknown_serial_' + str(unknown_serial_index)
                unknown_serial_index += 1
            else:
                device_serial = device_info['Serial']
            result[device_serial] = device_info
        return result

    def run(self):
        """
        Entry point.
        """
        self.devices = AdbWrapper.adb_devices()
        is_no_color = self.no_color
        if 'NO_COLOR' in os.environ:
            try:
                is_no_color = bool(util.strtobool(os.environ['NO_COLOR'].lower()))
            except Exception as e:
                logger.debug(e)
                logger.error('Invalid NO_COLOR value [{0}].'.format(os.environ['NO_COLOR']))

        if len(self.devices) == 0:
            raise Exception('No device.')
        elif len(self.devices) >= 1:
            final_serial = AdbHelper.get_serial(self.serial)
            if final_serial is None:
                self.device_info_list = []
                for device, state in self.devices.items():
                    print('Serial: {0} (State: {1})'.format(device, state))
                    if state == 'device':
                        device_info = self.get_device_info(serial=device)
                        self.print_device_info(device_info, no_color=is_no_color)
                        self.device_info_list.append(device_info)
                    else:
                        print('Skipped.\n')
                        self.device_info_list.append({'Serial': device, 'Skip': True})
            else:
                print('Serial: {0} (State: {1})'.format(final_serial, self.devices[final_serial]))
                device_info = self.get_device_info(serial=final_serial)
                self.device_info_list = [device_info]
                self.print_device_info(device_info, no_color=is_no_color)
            self._output_log()


def main():
    try:
        VersionChecker().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import re
import os
import shutil
import logging
import tempfile
import argparse
import ConfigParser
from argparse import ArgumentDefaultsHelpFormatter
from reset_phone import PhoneReseter
from check_versions import VersionChecker
from backup_restore_profile import BackupRestoreHelper
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper
from util.b2g_helper import B2GHelper
from util.decompressor import Decompressor

logger = logging.getLogger(__name__)


class ShallowFlashHelper(object):
    """
    Workaround for shallow flash Gaia or Gecko into device.
    """

    def __init__(self):
        # default settings
        self.serial = None
        self.gaia = None
        self.gecko = None
        self.keep_profile = False

    def set_serial(self, serial):
        """
        Setup the serial number.
        @param serial: the given serial number.
        """
        self.serial = serial
        logger.debug('Set serial: {}'.format(self.serial))

    def set_gaia(self, gaia):
        """
        Setup the Gaia package path.
        @param gaia: the given Gaia package path.
        """
        self.gaia = gaia
        logger.debug('Set gaia: {}'.format(self.gaia))

    def set_gecko(self, gecko):
        """
        Setup the Gecko package path.
        @param gecko: the given Gecko package path.
        """
        self.gecko = gecko
        logger.debug('Set gecko: {}'.format(self.gecko))

    def set_keep_profile(self, flag):
        """
        Setup the serial number.
        @param flag: True or False.
        """
        self.keep_profile = flag
        logger.debug('Set keep_profile: {}'.format(self.keep_profile))

    def cli(self):
        """
        Handle the argument parse, and the return the instance itself.
        """
        # argument parser
        arg_parser = argparse.ArgumentParser(
            description='Workaround for shallow flash Gaia or Gecko into device.',
            formatter_class=ArgumentDefaultsHelpFormatter)
        arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None,
                                help='Directs command to the device or emulator with the given serial number. '
                                     'Overrides ANDROID_SERIAL environment variable.')
        arg_parser.add_argument('-g', '--gaia', action='store', dest='gaia', default=None,
                                help='Specify the Gaia package. (zip format)')
        arg_parser.add_argument('-G', '--gecko', action='store', dest='gecko', default=None,
                                help='Specify the Gecko package. (tar.gz format)')
        arg_parser.add_argument('--keep-profile', action='store_true', dest='keep_profile', default=False,
                                help='Keep user profile of device. Only work with shallow flash Gaia. (BETA)')
        arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                                help='Turn on verbose output, with all the debug logger.')

        # parse args and setup the logging
        args = arg_parser.parse_args()
        # setup the logging config
        if args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        # check ADB
        AdbWrapper.check_adb()
        # assign the variable
        self.set_serial(args.serial)
        if args.gaia:
            if self._is_gaia_package(args.gaia):
                self.set_gaia(args.gaia)
        if args.gecko:
            if self._is_gecko_package(args.gecko):
                self.set_gecko(args.gecko)
        self.set_keep_profile(args.keep_profile)
        # return instance
        return self

    @staticmethod
    def _is_gaia_package(gaia_pkg):
        return os.path.isfile(gaia_pkg) and gaia_pkg.endswith('.zip')

    @staticmethod
    def _is_gecko_package(gecko_pkg):
        return os.path.isfile(gecko_pkg) and gecko_pkg.endswith('.tar.gz')

    def _clean_gaia(self):
        logger.info('Cleaning Gaia profile: Start')
        command_list = ['rm -r /cache/*',
                        'rm -r /data/b2g/*',
                        'rm -r /data/local/storage/persistent/*',
                        'rm -r /data/local/svoperapps',
                        'rm -r /data/local/webapps',
                        'rm -r /data/local/user.js',
                        'rm -r /data/local/permissions.sqlite*',
                        'rm -r /data/local/OfflineCache',
                        'rm -r /data/local/indexedDB',
                        'rm -r /data/local/debug_info_trigger',
                        'rm -r /system/b2g/webapps']
        for cmd in command_list:
            AdbWrapper.adb_shell(cmd, serial=self.serial)
        logger.info('Cleaning Gaia profile: Done')

    def _push_gaia(self, source_dir):
        # push into device
        logger.info('Pushing Gaia: Start')
        unziped_gaia_dir = os.path.join(source_dir, 'gaia')

        # push user.js
        source_user_pref_path = os.path.join(unziped_gaia_dir, 'profile', 'user.js')
        user_pref_path = os.path.join(source_dir, 'user.js')
        # prepare the user.js
        logger.debug('Prepare user.js file')
        with open(source_user_pref_path, 'r') as fin:
            with open(user_pref_path, 'w') as fout:
                for line in fin:
                    fout.write(line.replace('user_pref', 'pref'))
        user_pref_target_path = '/system/b2g/defaults/pref/'
        logger.info('push user.js...')
        logger.debug('adb push {} to {}'.format(user_pref_path, user_pref_target_path))
        AdbWrapper.adb_push(user_pref_path, user_pref_target_path, serial=self.serial)

        # push webapps
        webapps_path = os.path.join(unziped_gaia_dir, 'profile', 'webapps')
        webapps_target_path = '/system/b2g/webapps'
        logger.info('push webapps...')
        logger.debug('adb push {} to {}'.format(webapps_path, webapps_target_path))
        AdbWrapper.adb_push(webapps_path, webapps_target_path, serial=self.serial)

        # push settings.json
        settings_path = os.path.join(unziped_gaia_dir, 'profile', 'settings.json')
        settings_target_path = '/system/b2g/defaults/'
        logger.info('push settings.json...')
        logger.debug('adb push {} to {}'.format(settings_path, settings_target_path))
        AdbWrapper.adb_push(settings_path, settings_target_path, serial=self.serial)
        logger.info('Pushing Gaia: Done')

    def _backup_profile(self):
        """
        @return: backup profile's folder.
        """
        profile_dir = tempfile.mkdtemp(prefix='b2gprofile_')
        logger.debug('TEMP profile Folder: {}'.format(profile_dir))
        logger.info('Backup profile to [{}].'.format(profile_dir))
        backup = BackupRestoreHelper()
        backup.set_serial(self.serial)
        backup.set_backup(True)
        backup.set_no_reboot(True)
        backup.set_profile_dir(profile_dir)
        backup.run()
        #backup.backup_profile(local_dir=profile_dir, serial=self.serial)
        return profile_dir

    def _restore_profile(self, profile_dir):
        """
        @param profile_dir: the backup profile's folder.
        """
        if profile_dir:
            logger.info('Restore profile from [{}].'.format(profile_dir))
            backup = BackupRestoreHelper()
            backup.set_serial(self.serial)
            backup.set_restore(True)
            backup.set_no_reboot(True)
            backup.set_skip_version_check(True)
            backup.set_profile_dir(profile_dir)
            backup.run()

    def shallow_flash_gaia(self):
        """
        Shallow flash Gaia.
        """
        logger.info('Shallow flash Gaia: Start')
        tmp_dir = None
        try:
            # keep user profile only work with shallow flash Gaia
            profile_dir = None
            if self.keep_profile:
                profile_dir = self._backup_profile()
            # Create temp folder
            tmp_dir = tempfile.mkdtemp(prefix='shallowflash_')
            logger.debug('TEMP Gaia Folder: {}'.format(tmp_dir))
            # check the Gaia package
            if not self._is_gaia_package(self.gaia):
                raise Exception(
                    '[{}] is not Gaia package. Please check again.'.format(os.path.abspath(self.gaia)))
            # unzip Gaia zip to tmp
            Decompressor().unzip(self.gaia, tmp_dir)
            # clean and push gaia profile
            self._clean_gaia()
            self._push_gaia(tmp_dir)
            # retore profile
            if self.keep_profile:
                self._restore_profile(profile_dir)
        finally:
            if tmp_dir:
                logger.debug('Removing [{0}] folder...'.format(tmp_dir))
                shutil.rmtree(tmp_dir)
                logger.debug('TEMP Gaia Folder was removed: {}'.format(tmp_dir))
        logger.info('Shallow flash Gaia: Done')

    def _clean_gecko(self, source_dir):
        logger.info('Cleaning Gecko profile: Start')
        command_list = ['rm -r /system/media']
        for cmd in command_list:
            AdbWrapper.adb_shell(cmd, serial=self.serial)

        gecko_dir = os.path.join(source_dir, 'b2g')
        source_files = os.listdir(gecko_dir)
        logger.debug('The files which will push into device: {}'.format(source_files))
        adb_stdout, adb_retcode = AdbWrapper.adb_shell('ls /system/b2g/')
        device_files = adb_stdout.split()
        logger.debug('The files which on device /system/b2g/: {}'.format(device_files))
        removed_files = sorted(list(set(source_files + device_files)))
        removed_files.remove('defaults')
        removed_files.remove('webapps')
        logger.debug('Remove files list: {}'.format(removed_files))
        for file in removed_files:
            AdbWrapper.adb_shell('rm -r /system/b2g/{}'.format(file), serial=self.serial)

        logger.info('Cleaning Gecko profile: Done')

    def _push_gecko(self, source_dir):
        # push into device
        logger.info('Pushing Gecko: Start')
        gecko_dir = os.path.join(source_dir, 'b2g')
        # push
        target_path = '/system/b2g/'
        logger.debug('adb push {} to {}'.format(gecko_dir, target_path))
        AdbWrapper.adb_push(gecko_dir, target_path, serial=self.serial)
        # set excutable
        source_files = os.listdir(gecko_dir)
        executable_files = [os.path.join('/system/b2g/', f) for f in source_files if os.access(os.path.join(gecko_dir, f), os.X_OK)]
        logger.debug('Add executed permission on device: {}'.format(executable_files))
        for file in executable_files:
            AdbWrapper.adb_shell('chmod 777 {}'.format(file), serial=self.serial)
        logger.info('Pushing Gecko: Done')

    def shallow_flash_gecko(self):
        """
        Shallow flash Gecko.
        """
        logger.info('Shallow flash Gecko: Start')
        tmp_dir = None
        try:
            # Create temp folder
            tmp_dir = tempfile.mkdtemp(prefix='shallowflash_')
            logger.debug('TEMP Gecko Folder: {}'.format(tmp_dir))
            # check the Gaia package
            if not self._is_gecko_package(self.gecko):
                raise Exception(
                    '[{}] is not Gecko package. Please check again.'.format(os.path.abspath(self.gecko)))
            # unzip Gecko tar.gz to tmp
            Decompressor().untar(self.gecko, tmp_dir)
            # clean and push gecko profile
            self._clean_gecko(tmp_dir)
            self._push_gecko(tmp_dir)
        finally:
            if tmp_dir:
                logger.debug('Removing [{0}] folder...'.format(tmp_dir))
                shutil.rmtree(tmp_dir)
                logger.debug('TEMP Gecko Folder was removed: {}'.format(tmp_dir))
        logger.info('Shallow flash Gecko: Done')

    def prepare_step(self):
        # checking the adb root
        if not AdbWrapper.adb_root(serial=self.serial):
            raise Exception('No root permission for shallow flashing.')
        # checking the adb remount
        if not AdbWrapper.adb_remount(serial=self.serial):
            raise Exception('No permission to remount for shallow flashing.')
        # Stop B2G
        B2GHelper.stop_b2g(serial=self.serial)

    def final_step(self):
        if self.gaia and not self.keep_profile:
            # reset phone when flash gaia and not keep profile
            logger.info('Reset device after shallow flash the Gaia.')
            PhoneReseter().reset_phone(serial=self.serial)
        else:
            # adb shell reboot
            logger.info('Reboot device.')
            AdbWrapper.adb_shell('sync', serial=self.serial)
            AdbWrapper.adb_shell('reboot', serial=self.serial)
        # wait for device, and then check version
        AdbWrapper.adb_wait_for_device(timeout=120)
        logger.info('Check versions.')
        checker = VersionChecker()
        checker.set_serial(self.serial)
        checker.run()

    def run(self):
        """
        Entry point.
        """
        # get the device's serial number
        devices = AdbWrapper.adb_devices()
        if len(devices) == 0:
            raise Exception('No device.')
        else:
            self.serial = AdbHelper.get_serial(self.serial)
            if self.serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                logger.debug('Setup serial to [{0}]'.format(self.serial))

        if self.gaia or self.gecko:
            self.prepare_step()
            if self.serial:
                logger.info('Target device [{0}]'.format(self.serial))
            if self.gecko:
                self.shallow_flash_gecko()
            if self.gaia:
                self.shallow_flash_gaia()
            self.final_step()


def main():
    try:
        ShallowFlashHelper().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import sys
import stat
import shutil
import logging
import tarfile
import zipfile
import easygui
import tempfile
import textwrap
import argparse
from argparse import RawTextHelpFormatter
from sys import platform as _platform

from check_versions import VersionChecker
from shallow_flash import ShallowFlashHelper
from util.adb_helper import AdbWrapper
from util.decompressor import Decompressor
from taskcluster_util.taskcluster_traverse import TraverseRunner


logger = logging.getLogger(__name__)


class B2GTraverseRunner(TraverseRunner):

    def __init__(self, connection_options=None):
        self.has_image = False
        self.image_path = None
        self.image_magic_folder = 'b2g-distro/'

        self.has_gaia = False
        self.gaia_path = None
        self.gaia_magic_folder = 'gaia/profile/'

        self.has_gecko = False
        self.gecko_path = None
        self.gecko_magic_folder = 'b2g'

        super(B2GTraverseRunner, self).__init__(connection_options=connection_options)

    def parser(self):
        # argument parser
        parser = argparse.ArgumentParser(description='The simple GUI tool for flashing B2G from Taskcluster.',
                                         formatter_class=RawTextHelpFormatter,
                                         epilog=textwrap.dedent('''\
                                         For more information of Taskcluster, see:
                                         - http://docs.taskcluster.net/
                                         - https://pypi.python.org/pypi/taskcluster_util

                                         The tc_credentials.json Template:
                                             {
                                                 "clientId": "",
                                                 "accessToken": "",
                                                 "certificate": {
                                                     "version":1,
                                                     "scopes":["*"],
                                                     "start":xxx,
                                                     "expiry":xxx,
                                                     "seed":"xxx",
                                                     "signature":"xxx"
                                                 }
                                             }
                                         '''))
        parser.add_argument('--credentials', action='store', default=self.taskcluster_credentials, dest='credentials',
                            help='The credential JSON file\n(default: {})'.format(self.taskcluster_credentials))
        parser.add_argument('-n', '--namespace', action='store', dest='namespace', default='',
                            help='The namespace of task')
        parser.add_argument('-d', '--dest-dir', action='store', dest='dest_dir',
                            help='The dest folder (default: current working folder)')
        parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False,
                            help='Turn on verbose output, with all the debug logger.')
        return parser.parse_args(sys.argv[1:])

    def check_b2g_image(self, file_path):
        logger.debug('check image: {}'.format(file_path))
        if os.path.isfile(file_path) and file_path.endswith('.zip'):
            try:
                with zipfile.ZipFile(file_path, 'r') as f:
                    if self.image_magic_folder in f.namelist():
                        self.has_image = True
                        self.image_path = file_path
                        logger.info('Find B2G Image {}'.format(self.image_path))
                        return True
            except:
                logger.debug('cannot open: {}'.format(file_path))
                return False
        return False

    def check_gaia_package(self, file_path):
        logger.debug('check gaia: {}'.format(file_path))
        if os.path.isfile(file_path) and file_path.endswith('.zip'):
            try:
                with zipfile.ZipFile(file_path, 'r') as f:
                    if self.gaia_magic_folder in f.namelist():
                        self.has_gaia = True
                        self.gaia_path = file_path
                        logger.info('Find Gaia Package {}'.format(self.gaia_path))
                        return True
            except:
                logger.debug('cannot open: {}'.format(file_path))
                return False
        return False

    def check_gecko_package(self, file_path):
        logger.debug('check gecko: {}'.format(file_path))
        if os.path.isfile(file_path) and file_path.endswith('.tar.gz'):
            try:
                with tarfile.TarFile.open(file_path, 'r:gz') as f:
                    if self.gecko_magic_folder in f.getnames():
                        self.has_gecko = True
                        self.gecko_path = file_path
                        logger.info('Find Gecko Package {}'.format(self.gecko_path))
                        return True
            except:
                logger.debug('cannot open: {}'.format(file_path))
                return False
        return False

    def generate_flash_message(self):
        msg = 'What do you want to flash?\n\n' \
            '- Skip: do not want to flash any thing.\n'
        msg_footer = ''

        if _platform == "win32":
            logger.info('Flash B2G Image cannot run on Windows.')
        else:
            if self.has_image:
                msg = msg + '- Image: flash images into device.\n'
                msg_footer = msg_footer + '[Image] {}\n'.format(self.image_path)
        if self.has_gaia:
            msg = msg + '- Gaia: only shallow flash Gaia.\n'
            msg_footer = msg_footer + '[Gaia] {}\n'.format(self.gaia_path)
        if self.has_gecko:
            msg = msg + '- Gecko: only shallow flash Gecko.\n'
            msg_footer = msg_footer + '[Gecko] {}\n'.format(self.gecko_path)
        if self.has_gaia and self.has_gecko:
            msg = msg + '- Gaia_Gecko: shallow flash both Gaia and Gecko.\n'
        msg = msg + '\n' + msg_footer + '\nMake sure there is only ONE device connected to the computer!'
        return msg

    def generate_flash_choices(self):
        choices = ['Skip']
        if _platform == "win32":
            logger.info('Flash B2G Image cannot run on Windows.')
        else:
            if self.has_image:
                choices.append('Image')
        if self.has_gaia:
            choices.append('Gaia')
        if self.has_gecko:
            choices.append('Gecko')
        if self.has_gaia and self.has_gecko:
            choices.append('Gaia_Gecko')
        return choices

    def _flash_again(self):
        title = 'Flash Again?'
        msg = 'Would you like to flash next devcie with the same build?'
        return easygui.ynbox(msg, title)

    def flash_Skip(self):
        logger.info('Skip flash.')

    def flash_Image(self):
        logger.info('Run flash Image...')
        if self.image_path:
            self._flash_image(self.image_path)

    def _flash_image(self, image):
        try:
            self.temp_dir = tempfile.mkdtemp()
            logger.debug('Temporary folder: {}'.format(self.temp_dir))
            Decompressor().unzip(image, self.temp_dir)
            # set the permissions to rwxrwxr-x (509 in python's os.chmod)
            os.chmod(self.temp_dir + '/b2g-distro/flash.sh', stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
            os.chmod(self.temp_dir + '/b2g-distro/load-config.sh', stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
            while True:
                os.system('cd ' + self.temp_dir + '/b2g-distro; ./flash.sh -f')
                # wait for device, and then check version
                AdbWrapper.adb_wait_for_device(timeout=120)
                logger.info('Check versions.')
                checker = VersionChecker()
                checker.run()
                # flash more than one device
                if not self._flash_again():
                    break
        finally:
            try:
                shutil.rmtree(self.temp_dir)  # delete directory
            except OSError:
                logger.debug('Cannot remove temporary folder: {}'.format(self.temp_dir))

    def flash_Gaia(self):
        logger.info('Run flash Gaia...')
        if self.gaia_path:
            self._shallow_flash(gaia=self.gaia_path)

    def flash_Gecko(self):
        logger.info('Run flash Gecko...')
        if self.gecko_path:
            self._shallow_flash(gecko=self.gecko_path)

    def flash_Gaia_Gecko(self):
        logger.info('Run flash Gaia_Gecko...')
        if self.gaia_path and self.gecko_path:
            self._shallow_flash(gaia=self.gaia_path, gecko=self.gecko_path)

    def _shallow_flash(self, gaia=None, gecko=None):
        if gaia or gecko:
            logger.info('Shallow Flash...')
            while True:
                sfh = ShallowFlashHelper()
                if gaia:
                    logger.info('Gaia: {}'.format(gaia))
                    sfh.set_gaia(gaia)
                if gecko:
                    logger.info('Gecko: {}'.format(gecko))
                    sfh.set_gecko(gecko)
                title = 'Keep Profile'
                msg = 'Would you like to keep profile? (BETA)'
                keep_profile = easygui.ynbox(msg, title)
                if keep_profile:
                    logger.info('Keep Profile: {}'.format(keep_profile))
                    sfh.set_keep_profile(keep_profile)
                sfh.run()
                # flash more than one device
                if not self._flash_again():
                    break
        else:
            logger.warning('No Gaia and no Gecko for flashing.')

    def _reset_arguments(self):
        # reset image args
        self.has_image = False
        self.image_path = None
        # reset gaia args
        self.has_gaia = False
        self.gaia_path = None
        # reset gecko args
        self.has_gecko = False
        self.gecko_path = None
        logger.debug('Reset image/gaia/gecko path and flags.')
        super(B2GTraverseRunner, self)._reset_arguments()

    def do_after_download(self):
        """
        Do something after downloading finished.
        """
        # check the files
        for f in self.downloaded_file_list:
            if not self.check_b2g_image(os.path.abspath(f)):
                logger.debug('{} is not b2g image.'.format(f))
                if not self.check_gaia_package(os.path.abspath(f)):
                    logger.debug('{} is not gaia package.'.format(f))
                    if not self.check_gecko_package(os.path.abspath(f)):
                        logger.debug('{} is not gecko package.'.format(f))

        if (not self.has_image) and (not self.has_gaia) and (not self.has_gecko):
            # there is no any package found
            easygui.msgbox('Cannot found B2G Image, Gaia, and Gecko.', ok_button='I know')
        else:
            title = 'Flash B2G'
            msg = self.generate_flash_message()
            choices = self.generate_flash_choices()
            reply = easygui.buttonbox(msg, title, choices=choices)
            logger.info('Select: {}'.format(reply))
            # then call method by reply's string
            if reply:
                call_method = getattr(self, 'flash_' + reply)
                call_method()
        super(B2GTraverseRunner, self).do_after_download()


def main():
    try:
        B2GTraverseRunner().cli().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



import os
from setuptools import setup, find_packages


VERSION = '0.0.15'

install_requires = [
    'taskcluster_util==0.0.25',
]

here = os.path.dirname(os.path.abspath(__file__))
# get documentation from the README and HISTORY
try:
    with open(os.path.join(here, 'README.rst')) as doc:
        readme = doc.read()
except:
    readme = ''

try:
    with open(os.path.join(here, 'HISTORY.rst')) as doc:
        history = doc.read()
except:
    history = ''

long_description = readme + '\n\n' + history

if __name__ == '__main__':
    setup(
        name='b2g_util',
        version=VERSION,
        description='B2G Utilities',
        long_description=long_description,
        keywords='mozilla b2g firefoxos fxos firefox utilities ',
        author='Askeing Yen',
        author_email='askeing@gmail.com',
        url='https://github.com/askeing/b2g-util-python',
        packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
        package_data={},
        install_requires=install_requires,
        zip_safe=False,
        entry_points="""
        # -*- Entry points: -*-
        [console_scripts]
        b2g_backup_restore_profile = b2g_util.backup_restore_profile:main
        b2g_check_versions = b2g_util.check_versions:main
        b2g_enable_certapps_devtools = b2g_util.enable_certapps_devtools:main
        b2g_flash_taskcluster = b2g_util.flash_taskcluster:main
        b2g_get_crashreports = b2g_util.get_crashreports:main
        b2g_reset_phone = b2g_util.reset_phone:main
        b2g_shallow_flash = b2g_util.shallow_flash:main
        """,
    )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import types
import logging
import tarfile
import zipfile


logger = logging.getLogger(__name__)


class Decompressor(object):

    @classmethod
    def unzip(cls, source_file, dest_folder):
        try:
            logger.info('Unzip {} to {}'.format(source_file, dest_folder))
            zip_file = zipfile.ZipFile(source_file)
            zip_file.extractall(dest_folder)
            zip_file.close()
            logger.info('Unzip done')
        except Exception as e:
            logger.error('Unzip {} Error'.format(source_file))

    @classmethod
    def untar(cls, source_file, dest_folder):
        try:
            logger.info('Untar {} to {}'.format(source_file, dest_folder))
            tar_file = tarfile.open(source_file)
            tar_file.extractall(dest_folder)
            tar_file.close()
            logger.info('Untar done')
        except Exception as e:
            logger.error('Unzip {} Error'.format(source_file))

    @classmethod
    def ensure_folder(cls, folder):
        if not os.path.isdir(folder):
            os.makedirs(folder)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import re
import logging
import threading
import subprocess
from distutils import spawn


logger = logging.getLogger(__name__)


class AdbWrapper(object):

    @classmethod
    def check_adb(cls):
        """
        Check the ADB command.
        @return: True if your system have ADB command.
        @raise exception: There is no ADB command in your system.
        """
        logger.debug('Checking ADB...')
        if spawn.find_executable('adb') is None:
            raise Exception('There is no "adb" in your environment PATH.')
        logger.debug('You have ADB.')
        return True

    @classmethod
    def adb_devices(cls):
        """
        Get the device list.
        @return: devices as dict {device_serial: device_status, ...}.
        @raise exception: When return code isn't zero.
        """
        cmd = 'adb devices'
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))
        output = re.sub(r'\r+', '', output)
        output_list = re.split(r'\n+', output)
        # remove '', '* daemon not running. starting it now on port xxx *', and '* daemon started successfully *'
        output_list = [item for item in output_list if item and not item.startswith('* daemon')]
        msg_filter = re.compile(r'(^List of devices attached\s*|^\s+$)')
        output_list = [i for i in output_list if not msg_filter.search(i)]
        output_list = [(re.split(r'\t', item)) for item in output_list if True]
        devices = {}
        for device in output_list:
            devices[device[0]] = device[1]
        return devices

    @classmethod
    def adb_pull(cls, source, dest, serial=None):
        """
        Pull files from device.
        @return: stdout of command.
        @raise exception: When return code isn't zero.
        """
        if serial is None:
            cmd = 'adb pull'
        else:
            cmd = 'adb -s %s pull' % (serial,)
        cmd = "%s '%s' '%s'" % (cmd, source, dest)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))
        return output

    @classmethod
    def adb_push(cls, source, dest, serial=None):
        """
        Push files into device.
        @return: stdout of command.
        @raise exception: when return code isn't zero.
        """
        if serial is None:
            cmd = 'adb push'
        else:
            cmd = 'adb -s %s push' % (serial,)
        cmd = "%s '%s' '%s'" % (cmd, source, dest)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))
        return output

    @classmethod
    def adb_shell(cls, command, serial=None):
        """
        Run command on device.
        @return: the stdout and return code (from device) of "adb shell" command. e.g. (stdout, retcode)
        @raise exception: When return code (from adb command) isn't zero.
        """
        if serial is None:
            cmd = 'adb shell'
        else:
            cmd = 'adb -s %s shell' % (serial,)
        # get returncode from device
        cmd = "%s '%s; echo $?'" % (cmd, command)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        shell_ret, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RAW_RET: {0}'.format(shell_ret))
        if stderr:
            logger.debug('RAW_ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': shell_ret, 'STDERR': stderr}))
        # split the stdout and retcode (from device)
        shell_ret = re.sub(r'\s+$', '', shell_ret)
        shell_output = re.split(r'\s+(\d+$)', shell_ret, maxsplit=1)
        # when no stdout, set output to ''
        if len(shell_output) == 1:
            # only returncode, no stdout
            output = ''
            try:
                returncode = int(shell_output[0])
            except Exception as e:
                logger.debug(e)
                # some command will stop device with no returncode. e.g. adb shell reboot recovery
                returncode = 0
        else:
            # stdout with returncode
            output = shell_output[0]
            try:
                returncode = int(shell_output[1])
            except Exception as e:
                logger.debug(e)
                returncode = 0
        logger.debug('RET: {0}'.format(output))
        logger.debug('RET CODE: {0}'.format(returncode))
        return output, returncode

    @classmethod
    def adb_root(cls, serial=None):
        """
        Get the root permission of ADB.
        @return: True if adb already running as root. False if failed.
        """
        if serial is None:
            cmd = 'adb root'
        else:
            cmd = 'adb -s %s root' % (serial,)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is 0 and ('cannot' not in output):
            if 'restarting' in output:
                import time
                time.sleep(1)
            logger.debug('adb root successed')
            logger.info('{}'.format(output))
            return True
        else:
            logger.debug('adb root failed')
            logger.warning('{}'.format(output))
            return False

    @classmethod
    def adb_remount(cls, serial=None):
        """
        Remounts the /system partition on the device read-write
        @return: True if succeeded. False if failed.
        """
        if serial is None:
            cmd = 'adb remount'
        else:
            cmd = 'adb -s %s remount' % (serial,)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is 0:
            if 'remount succeeded' in output:
                logger.info('{}'.format(output))
                return True
            else:
                logger.warning('{}'.format(output))
                return False
        else:
            logger.debug('adb remount failed')
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))

    @classmethod
    def adb_wait_for_device(cls, timeout=60, serial=None):
        """
        block until device is online. (default timeout is 60 seconds)
        @param timeout: specify the timeout for the operation in seconds. Default is 60 seconds.
        @raise exception: when running for more than timeout seconds.
        """
        thread = None
        cls.p_wait_for_device = None

        def _wait_for_device():
            logger.info('Starting adb wait-for-device, timeout: {}, serial: {}'.format(timeout, serial))
            if serial is None:
                cmd = 'adb wait-for-device'
            else:
                cmd = 'adb -s %s wait-for-device' % (serial,)
            cls.p_wait_for_device = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            cls.p_wait_for_device.communicate()
        thread = threading.Thread(target=_wait_for_device)
        thread.start()
        thread.join(timeout)
        if thread.is_alive():
            logger.debug('Terminating wait-for-device process.')
            cls.p_wait_for_device.terminate()
            cls.p_wait_for_device = None
            raise Exception('adb wait-for-device timeout, timeout {}, serial: {}'.format(timeout, serial))
        return True


class AdbHelper(object):

    @classmethod
    def get_serial(cls, serial_number):
        """
        Input the serial number.
        This method will check with "ANDROID_SERIAL" environmental variable,
        and then return the serial number if the serial is avaiable.
        When serial number is not avaiable, return None.

        ex:
          - (serial O, ANDROID_SERIAL O) => serial
          - (serial O, ANDROID_SERIAL X) => serial
          - (serial X, ANDROID_SERIAL O) => ANDROID_SERIAL
          - (serial X, ANDROID_SERIAL X) => None

        @return: the serial number of device.
        @raise exception: if there is no device have the given serial number.
        """
        # return None if there are no serial and ANDROID_SERIAL
        final_serial_number = None
        # no serial then check ANDROID_SERIAL
        if serial_number is None or serial_number == '':
            if 'ANDROID_SERIAL' in os.environ:
                logger.debug('ANDROID_SERIAL={}'.format(os.environ.get('ANDROID_SERIAL')))
                final_serial_number = os.environ.get('ANDROID_SERIAL')
        else:
            final_serial_number = serial_number
            logger.debug('serial={}'.format(serial_number))
        # raise Exception if the serial is not in devices list
        if final_serial_number is not None:
            devices = AdbWrapper.adb_devices()
            logger.debug('Devices: {}'.format(devices))
            if final_serial_number not in devices:
                raise Exception('Can not found {} device in devices list {}.'.format(final_serial_number, devices))
        return final_serial_number



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import sys
import string
import getpass
import logging


logger = logging.getLogger(__name__)


class ConsoleDialog(object):

    _SYMBOL_CHAR = '#'
    _NAME = 'NAME'
    _TYPE = 'TYPE'
    _COMMAND_TYPE = 'COMMAND'
    _ITEM_TYPE = 'ITEM'
    _QUIT_CMD_INDEX = 'q'
    _QUIT_CMD_NAME = 'QUIT'
    _YES_CMD_INDEX = 'y'
    _NO_CMD_INDEX = 'n'

    def __init__(self):
        """
        Setup the console rows and columns. Default is 23x80.
        """
        try:
            self.console_rows, self.console_columns = ConsoleDialog.get_terminal_size()
        except Exception as e:
            logger.debug(e)
            self.console_rows = 23
            self.console_columns = 80

    def menu(self, title, description, items_list, alternative_cmds_dict=None):
        """
        Create the menu dialog.

        The index of items_list will be numbers.

        The index of alternative_cmds_dict should be lowercase characters. ex: {'b': 'BACK TO PREV STEP', 'h': 'HELP'}

        @param title: The title.
        @param description: The description.
        @param items_list: The input item list. e.g. {1: 'foo', 2: 'bar'}.
        @param alternative_cmds_dict: The command list. e.g. {1: 'foo', 2: 'bar'}.
        @return: {'SELECT': USER_INPUT, 'ITEMS': ALL_ITEMS_DICT}.
        """
        if not alternative_cmds_dict:
            alternative_cmds_dict = {}
        # print title
        self._print_title(title)
        # create items dict
        items_dict = self._create_items_dict(items_list, alternative_cmds_dict)
        self._print_items_dict(items_dict)
        # get response from raw input
        response = ''
        while response not in items_dict.keys() and not response == self._QUIT_CMD_INDEX:
            response = self._get_dialog_input_with_desc(description).lower()
        # return selection
        return {'SELECT': response, 'ITEMS': items_dict}

    def input_box(self, title, description, password=False):
        """
        Create the input dialog.
        @param title: The title.
        @param description: The description.
        @param password: Do not display the input when type is password, default is False.
        @return: The input string from user.
        """
        # print title
        self._print_title(title)
        if password:
            response = getpass.getpass('> ' + description + ': ')
        else:
            response = self._get_dialog_input_with_desc(description)
        return response

    def msg_box(self, title, description, press_enter_to_next=False):
        """
        Create the message dialog.
        @param title: The title.
        @param description: The description.
        @param press_enter_to_next: Waiting for pressing Enter, default is False.
        """
        # print title
        self._print_title(title)
        print '> ' + description
        self._print_horizontal_lines()
        print ''
        if press_enter_to_next:
            raw_input("Press Enter to continue...")

    def yes_no(self, title, description, default_value=''):
        """
        Create the Yes/No dialog.
        @param title: The title.
        @param description: The description.
        @param default_value: The default value, 'y' or 'n'. Default is ''.
        @return: True or False from user.
        """
        question_mark = ' (y/n) '
        if default_value == self._YES_CMD_INDEX:
            question_mark = ' (Y/n) '
        elif default_value == self._NO_CMD_INDEX:
            question_mark = ' (y/N) '
        # print title
        self._print_title(title)
        # get response from raw input
        response = ''
        while response not in [self._YES_CMD_INDEX, self._NO_CMD_INDEX]:
            response = self._get_dialog_input_with_desc(description + question_mark).lower()
            if len(default_value) > 0 and response not in [self._YES_CMD_INDEX, self._NO_CMD_INDEX]:
                response = default_value
        answer = True if response == self._YES_CMD_INDEX else False
        return answer

    def _create_items_dict(self, items_list, alternative_cmds_dict=None):
        if not alternative_cmds_dict:
            alternative_cmds_dict = {}
        items_dict = {self._QUIT_CMD_INDEX: {self._NAME: self._QUIT_CMD_NAME, self._TYPE: self._COMMAND_TYPE}}
        # create the items dict.
        index = 1
        for item in items_list:
            items_dict[str(index)] = {self._NAME: item, self._TYPE: self._ITEM_TYPE}
            index += 1
        # create the alternative options dict.
        if len(alternative_cmds_dict) > 0:
            temp_alternative_options_dict = alternative_cmds_dict.copy()
            option_index = 0
            # if the option key still not be used, then add it into result dict.
            for option_key, option_value in temp_alternative_options_dict.items():
                if option_key not in items_dict.keys():
                    items_dict[str(option_key)] = {self._NAME: option_value, self._TYPE: self._COMMAND_TYPE}
                    del temp_alternative_options_dict[option_key]
            # if the option key be used, then add it with no used char.
            for option_key, option_value in temp_alternative_options_dict.items():
                chars = string.lowercase.replace('q', '')
                while chars[option_index] in items_dict.keys():
                    option_index += 1
                items_dict[chars[option_index]] = {self._NAME: option_value, self._TYPE: self._COMMAND_TYPE}
        return items_dict

    def _print_items_dict(self, items_dict):
        def _key2int(x):
            try:
                return int(x)
            except Exception as e:
                logger.debug(e)
                return x
        for index in sorted(items_dict, key=_key2int):
            if items_dict[index][self._TYPE] == self._ITEM_TYPE:
                print 'ITEM',
            else:
                print 'CMD ',
            print index + ')', items_dict[index][self._NAME]
        self._print_horizontal_lines()

    def _print_title(self, title):
        print ''
        self._update_terminal_size()
        self._print_horizontal_lines()
        print self._SYMBOL_CHAR, title
        self._print_horizontal_lines()

    @staticmethod
    def _get_dialog_input_with_desc(description):
        response = raw_input('> ' + description + ': ')
        return response

    def _print_horizontal_lines(self):
        for i in xrange(0, int(self.console_columns) - 1):
            sys.stdout.write(self._SYMBOL_CHAR)
            sys.stdout.flush()
        sys.stdout.write('\n')

    def _update_terminal_size(self):
        try:
            self.console_rows, self.console_columns = ConsoleDialog.get_terminal_size()
        except Exception as e:
            logger.debug(e)
            self.console_rows = 23
            self.console_columns = 80

    @staticmethod
    def get_terminal_size():
        """
        @return: the rows and columns of terminal.
        @see: U{http://stackoverflow.com/questions/566746/how-to-get-console-window-width-in-python}
        """
        return os.popen('stty size', 'r').read().split()
